# SPDX-FileCopyrightText: 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

# ruff: noqa:E501

from pontos.github.api.secret_scanning import GitHubAsyncRESTSecretScanning
from pontos.github.models.base import SortOrder
from pontos.github.models.secret_scanning import (
    AlertSort,
    AlertState,
    LocationType,
    Resolution,
)
from tests import AsyncIteratorMock, aiter, anext
from tests.github.api import GitHubAsyncRESTTestCase, create_response

ALERTS = [
    {
        "number": 2,
        "created_at": "2020-11-06T18:48:51Z",
        "url": "https://api.github.com/repos/owner/private-repo/secret-scanning/alerts/2",
        "html_url": "https://github.com/owner/private-repo/security/secret-scanning/2",
        "locations_url": "https://api.github.com/repos/owner/private-repo/secret-scanning/alerts/2/locations",
        "state": "resolved",
        "resolution": "false_positive",
        "resolved_at": "2020-11-07T02:47:13Z",
        "resolved_by": {
            "login": "monalisa",
            "id": 2,
            "node_id": "MDQ6VXNlcjI=",
            "avatar_url": "https://alambic.github.com/avatars/u/2?",
            "gravatar_id": "",
            "url": "https://api.github.com/users/monalisa",
            "html_url": "https://github.com/monalisa",
            "followers_url": "https://api.github.com/users/monalisa/followers",
            "following_url": "https://api.github.com/users/monalisa/following{/other_user}",
            "gists_url": "https://api.github.com/users/monalisa/gists{/gist_id}",
            "starred_url": "https://api.github.com/users/monalisa/starred{/owner}{/repo}",
            "subscriptions_url": "https://api.github.com/users/monalisa/subscriptions",
            "organizations_url": "https://api.github.com/users/monalisa/orgs",
            "repos_url": "https://api.github.com/users/monalisa/repos",
            "events_url": "https://api.github.com/users/monalisa/events{/privacy}",
            "received_events_url": "https://api.github.com/users/monalisa/received_events",
            "type": "User",
            "site_admin": True,
        },
        "secret_type": "adafruit_io_key",
        "secret_type_display_name": "Adafruit IO Key",
        "secret": "aio_XXXXXXXXXXXXXXXXXXXXXXXXXXXX",
        "repository": {
            "id": 1296269,
            "node_id": "MDEwOlJlcG9zaXRvcnkxMjk2MjY5",
            "name": "Hello-World",
            "full_name": "octocat/Hello-World",
            "owner": {
                "login": "octocat",
                "id": 1,
                "node_id": "MDQ6VXNlcjE=",
                "avatar_url": "https://github.com/images/error/octocat_happy.gif",
                "gravatar_id": "",
                "url": "https://api.github.com/users/octocat",
                "html_url": "https://github.com/octocat",
                "followers_url": "https://api.github.com/users/octocat/followers",
                "following_url": "https://api.github.com/users/octocat/following{/other_user}",
                "gists_url": "https://api.github.com/users/octocat/gists{/gist_id}",
                "starred_url": "https://api.github.com/users/octocat/starred{/owner}{/repo}",
                "subscriptions_url": "https://api.github.com/users/octocat/subscriptions",
                "organizations_url": "https://api.github.com/users/octocat/orgs",
                "repos_url": "https://api.github.com/users/octocat/repos",
                "events_url": "https://api.github.com/users/octocat/events{/privacy}",
                "received_events_url": "https://api.github.com/users/octocat/received_events",
                "type": "User",
                "site_admin": False,
            },
            "private": False,
            "html_url": "https://github.com/octocat/Hello-World",
            "description": "This your first repo!",
            "fork": False,
            "url": "https://api.github.com/repos/octocat/Hello-World",
            "archive_url": "https://api.github.com/repos/octocat/Hello-World/{archive_format}{/ref}",
            "assignees_url": "https://api.github.com/repos/octocat/Hello-World/assignees{/user}",
            "blobs_url": "https://api.github.com/repos/octocat/Hello-World/git/blobs{/sha}",
            "branches_url": "https://api.github.com/repos/octocat/Hello-World/branches{/branch}",
            "collaborators_url": "https://api.github.com/repos/octocat/Hello-World/collaborators{/collaborator}",
            "comments_url": "https://api.github.com/repos/octocat/Hello-World/comments{/number}",
            "commits_url": "https://api.github.com/repos/octocat/Hello-World/commits{/sha}",
            "compare_url": "https://api.github.com/repos/octocat/Hello-World/compare/{base}...{head}",
            "contents_url": "https://api.github.com/repos/octocat/Hello-World/contents/{+path}",
            "contributors_url": "https://api.github.com/repos/octocat/Hello-World/contributors",
            "deployments_url": "https://api.github.com/repos/octocat/Hello-World/deployments",
            "downloads_url": "https://api.github.com/repos/octocat/Hello-World/downloads",
            "events_url": "https://api.github.com/repos/octocat/Hello-World/events",
            "forks_url": "https://api.github.com/repos/octocat/Hello-World/forks",
            "git_commits_url": "https://api.github.com/repos/octocat/Hello-World/git/commits{/sha}",
            "git_refs_url": "https://api.github.com/repos/octocat/Hello-World/git/refs{/sha}",
            "git_tags_url": "https://api.github.com/repos/octocat/Hello-World/git/tags{/sha}",
            "issue_comment_url": "https://api.github.com/repos/octocat/Hello-World/issues/comments{/number}",
            "issue_events_url": "https://api.github.com/repos/octocat/Hello-World/issues/events{/number}",
            "issues_url": "https://api.github.com/repos/octocat/Hello-World/issues{/number}",
            "keys_url": "https://api.github.com/repos/octocat/Hello-World/keys{/key_id}",
            "labels_url": "https://api.github.com/repos/octocat/Hello-World/labels{/name}",
            "languages_url": "https://api.github.com/repos/octocat/Hello-World/languages",
            "merges_url": "https://api.github.com/repos/octocat/Hello-World/merges",
            "milestones_url": "https://api.github.com/repos/octocat/Hello-World/milestones{/number}",
            "notifications_url": "https://api.github.com/repos/octocat/Hello-World/notifications{?since,all,participating}",
            "pulls_url": "https://api.github.com/repos/octocat/Hello-World/pulls{/number}",
            "releases_url": "https://api.github.com/repos/octocat/Hello-World/releases{/id}",
            "stargazers_url": "https://api.github.com/repos/octocat/Hello-World/stargazers",
            "statuses_url": "https://api.github.com/repos/octocat/Hello-World/statuses/{sha}",
            "subscribers_url": "https://api.github.com/repos/octocat/Hello-World/subscribers",
            "subscription_url": "https://api.github.com/repos/octocat/Hello-World/subscription",
            "tags_url": "https://api.github.com/repos/octocat/Hello-World/tags",
            "teams_url": "https://api.github.com/repos/octocat/Hello-World/teams",
            "trees_url": "https://api.github.com/repos/octocat/Hello-World/git/trees{/sha}",
            "hooks_url": "https://api.github.com/repos/octocat/Hello-World/hooks",
        },
        "push_protection_bypassed_by": {
            "login": "monalisa",
            "id": 2,
            "node_id": "MDQ6VXNlcjI=",
            "avatar_url": "https://alambic.github.com/avatars/u/2?",
            "gravatar_id": "",
            "url": "https://api.github.com/users/monalisa",
            "html_url": "https://github.com/monalisa",
            "followers_url": "https://api.github.com/users/monalisa/followers",
            "following_url": "https://api.github.com/users/monalisa/following{/other_user}",
            "gists_url": "https://api.github.com/users/monalisa/gists{/gist_id}",
            "starred_url": "https://api.github.com/users/monalisa/starred{/owner}{/repo}",
            "subscriptions_url": "https://api.github.com/users/monalisa/subscriptions",
            "organizations_url": "https://api.github.com/users/monalisa/orgs",
            "repos_url": "https://api.github.com/users/monalisa/repos",
            "events_url": "https://api.github.com/users/monalisa/events{/privacy}",
            "received_events_url": "https://api.github.com/users/monalisa/received_events",
            "type": "User",
            "site_admin": True,
        },
        "push_protection_bypassed": True,
        "push_protection_bypassed_at": "2020-11-06T21:48:51Z",
        "resolution_comment": "Example comment",
    },
    {
        "number": 1,
        "created_at": "2020-11-06T18:18:30Z",
        "url": "https://api.github.com/repos/owner/repo/secret-scanning/alerts/1",
        "html_url": "https://github.com/owner/repo/security/secret-scanning/1",
        "locations_url": "https://api.github.com/repos/owner/private-repo/secret-scanning/alerts/1/locations",
        "state": "open",
        "resolution": None,
        "resolved_at": None,
        "resolved_by": None,
        "secret_type": "mailchimp_api_key",
        "secret_type_display_name": "Mailchimp API Key",
        "secret": "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX-us2",
        "repository": {
            "id": 1296269,
            "node_id": "MDEwOlJlcG9zaXRvcnkxMjk2MjY5",
            "name": "Hello-World",
            "full_name": "octocat/Hello-World",
            "owner": {
                "login": "octocat",
                "id": 1,
                "node_id": "MDQ6VXNlcjE=",
                "avatar_url": "https://github.com/images/error/octocat_happy.gif",
                "gravatar_id": "",
                "url": "https://api.github.com/users/octocat",
                "html_url": "https://github.com/octocat",
                "followers_url": "https://api.github.com/users/octocat/followers",
                "following_url": "https://api.github.com/users/octocat/following{/other_user}",
                "gists_url": "https://api.github.com/users/octocat/gists{/gist_id}",
                "starred_url": "https://api.github.com/users/octocat/starred{/owner}{/repo}",
                "subscriptions_url": "https://api.github.com/users/octocat/subscriptions",
                "organizations_url": "https://api.github.com/users/octocat/orgs",
                "repos_url": "https://api.github.com/users/octocat/repos",
                "events_url": "https://api.github.com/users/octocat/events{/privacy}",
                "received_events_url": "https://api.github.com/users/octocat/received_events",
                "type": "User",
                "site_admin": False,
            },
            "private": False,
            "html_url": "https://github.com/octocat/Hello-World",
            "description": "This your first repo!",
            "fork": False,
            "url": "https://api.github.com/repos/octocat/Hello-World",
            "archive_url": "https://api.github.com/repos/octocat/Hello-World/{archive_format}{/ref}",
            "assignees_url": "https://api.github.com/repos/octocat/Hello-World/assignees{/user}",
            "blobs_url": "https://api.github.com/repos/octocat/Hello-World/git/blobs{/sha}",
            "branches_url": "https://api.github.com/repos/octocat/Hello-World/branches{/branch}",
            "collaborators_url": "https://api.github.com/repos/octocat/Hello-World/collaborators{/collaborator}",
            "comments_url": "https://api.github.com/repos/octocat/Hello-World/comments{/number}",
            "commits_url": "https://api.github.com/repos/octocat/Hello-World/commits{/sha}",
            "compare_url": "https://api.github.com/repos/octocat/Hello-World/compare/{base}...{head}",
            "contents_url": "https://api.github.com/repos/octocat/Hello-World/contents/{+path}",
            "contributors_url": "https://api.github.com/repos/octocat/Hello-World/contributors",
            "deployments_url": "https://api.github.com/repos/octocat/Hello-World/deployments",
            "downloads_url": "https://api.github.com/repos/octocat/Hello-World/downloads",
            "events_url": "https://api.github.com/repos/octocat/Hello-World/events",
            "forks_url": "https://api.github.com/repos/octocat/Hello-World/forks",
            "git_commits_url": "https://api.github.com/repos/octocat/Hello-World/git/commits{/sha}",
            "git_refs_url": "https://api.github.com/repos/octocat/Hello-World/git/refs{/sha}",
            "git_tags_url": "https://api.github.com/repos/octocat/Hello-World/git/tags{/sha}",
            "issue_comment_url": "https://api.github.com/repos/octocat/Hello-World/issues/comments{/number}",
            "issue_events_url": "https://api.github.com/repos/octocat/Hello-World/issues/events{/number}",
            "issues_url": "https://api.github.com/repos/octocat/Hello-World/issues{/number}",
            "keys_url": "https://api.github.com/repos/octocat/Hello-World/keys{/key_id}",
            "labels_url": "https://api.github.com/repos/octocat/Hello-World/labels{/name}",
            "languages_url": "https://api.github.com/repos/octocat/Hello-World/languages",
            "merges_url": "https://api.github.com/repos/octocat/Hello-World/merges",
            "milestones_url": "https://api.github.com/repos/octocat/Hello-World/milestones{/number}",
            "notifications_url": "https://api.github.com/repos/octocat/Hello-World/notifications{?since,all,participating}",
            "pulls_url": "https://api.github.com/repos/octocat/Hello-World/pulls{/number}",
            "releases_url": "https://api.github.com/repos/octocat/Hello-World/releases{/id}",
            "stargazers_url": "https://api.github.com/repos/octocat/Hello-World/stargazers",
            "statuses_url": "https://api.github.com/repos/octocat/Hello-World/statuses/{sha}",
            "subscribers_url": "https://api.github.com/repos/octocat/Hello-World/subscribers",
            "subscription_url": "https://api.github.com/repos/octocat/Hello-World/subscription",
            "tags_url": "https://api.github.com/repos/octocat/Hello-World/tags",
            "teams_url": "https://api.github.com/repos/octocat/Hello-World/teams",
            "trees_url": "https://api.github.com/repos/octocat/Hello-World/git/trees{/sha}",
            "hooks_url": "https://api.github.com/repos/octocat/Hello-World/hooks",
        },
        "push_protection_bypassed_by": None,
        "push_protection_bypassed": False,
        "push_protection_bypassed_at": None,
        "resolution_comment": None,
    },
]


class GitHubAsyncRESTSecretScanningTestCase(GitHubAsyncRESTTestCase):
    api_cls = GitHubAsyncRESTSecretScanning

    async def test_enterprise_alerts(self):
        response = create_response()
        response.json.return_value = ALERTS

        self.client.get_all.return_value = AsyncIteratorMock([response])

        async_it = aiter(self.api.enterprise_alerts("foo"))
        alert = await anext(async_it)
        self.assertEqual(alert.number, 2)
        alert = await anext(async_it)
        self.assertEqual(alert.number, 1)

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/enterprises/foo/secret-scanning/alerts",
            params={"per_page": "100", "sort": "created", "direction": "desc"},
        )

    async def test_enterprise_alerts_state(self):
        response = create_response()
        response.json.return_value = ALERTS

        self.client.get_all.return_value = AsyncIteratorMock([response])

        async_it = aiter(
            self.api.enterprise_alerts("foo", state=AlertState.RESOLVED)
        )
        alert = await anext(async_it)
        self.assertEqual(alert.number, 2)
        alert = await anext(async_it)
        self.assertEqual(alert.number, 1)

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/enterprises/foo/secret-scanning/alerts",
            params={
                "per_page": "100",
                "sort": "created",
                "direction": "desc",
                "state": "resolved",
            },
        )

    async def test_enterprise_alerts_secret_types(self):
        response = create_response()
        response.json.return_value = ALERTS

        self.client.get_all.return_value = AsyncIteratorMock([response])

        async_it = aiter(
            self.api.enterprise_alerts(
                "foo",
                secret_types=[
                    "google_api_key",
                    "hashicorp_vault_service_token",
                ],
            )
        )
        alert = await anext(async_it)
        self.assertEqual(alert.number, 2)
        alert = await anext(async_it)
        self.assertEqual(alert.number, 1)

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/enterprises/foo/secret-scanning/alerts",
            params={
                "per_page": "100",
                "sort": "created",
                "direction": "desc",
                "secret_type": "google_api_key,hashicorp_vault_service_token",
            },
        )

    async def test_enterprise_alerts_resolutions(self):
        response = create_response()
        response.json.return_value = ALERTS

        self.client.get_all.return_value = AsyncIteratorMock([response])

        async_it = aiter(
            self.api.enterprise_alerts(
                "foo",
                resolutions=["false_positive", "wont_fix", "revoked"],
            )
        )
        alert = await anext(async_it)
        self.assertEqual(alert.number, 2)
        alert = await anext(async_it)
        self.assertEqual(alert.number, 1)

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/enterprises/foo/secret-scanning/alerts",
            params={
                "per_page": "100",
                "sort": "created",
                "direction": "desc",
                "resolution": "false_positive,wont_fix,revoked",
            },
        )

    async def test_enterprise_alerts_sort(self):
        response = create_response()
        response.json.return_value = ALERTS

        self.client.get_all.return_value = AsyncIteratorMock([response])

        async_it = aiter(
            self.api.enterprise_alerts("foo", sort=AlertSort.UPDATED)
        )
        alert = await anext(async_it)
        self.assertEqual(alert.number, 2)
        alert = await anext(async_it)
        self.assertEqual(alert.number, 1)

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/enterprises/foo/secret-scanning/alerts",
            params={
                "per_page": "100",
                "sort": "updated",
                "direction": "desc",
            },
        )

    async def test_enterprise_alerts_direction(self):
        response = create_response()
        response.json.return_value = ALERTS

        self.client.get_all.return_value = AsyncIteratorMock([response])

        async_it = aiter(
            self.api.enterprise_alerts("foo", direction=SortOrder.ASC)
        )
        alert = await anext(async_it)
        self.assertEqual(alert.number, 2)
        alert = await anext(async_it)
        self.assertEqual(alert.number, 1)

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/enterprises/foo/secret-scanning/alerts",
            params={
                "per_page": "100",
                "sort": "created",
                "direction": "asc",
            },
        )

    async def test_organization_alerts(self):
        response = create_response()
        response.json.return_value = ALERTS

        self.client.get_all.return_value = AsyncIteratorMock([response])

        async_it = aiter(self.api.organization_alerts("foo"))
        alert = await anext(async_it)
        self.assertEqual(alert.number, 2)
        alert = await anext(async_it)
        self.assertEqual(alert.number, 1)

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/orgs/foo/secret-scanning/alerts",
            params={"per_page": "100", "sort": "created", "direction": "desc"},
        )

    async def test_organization_alerts_state(self):
        response = create_response()
        response.json.return_value = ALERTS

        self.client.get_all.return_value = AsyncIteratorMock([response])

        async_it = aiter(
            self.api.organization_alerts("foo", state=AlertState.RESOLVED)
        )
        alert = await anext(async_it)
        self.assertEqual(alert.number, 2)
        alert = await anext(async_it)
        self.assertEqual(alert.number, 1)

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/orgs/foo/secret-scanning/alerts",
            params={
                "per_page": "100",
                "sort": "created",
                "direction": "desc",
                "state": "resolved",
            },
        )

    async def test_organization_alerts_secret_types(self):
        response = create_response()
        response.json.return_value = ALERTS

        self.client.get_all.return_value = AsyncIteratorMock([response])

        async_it = aiter(
            self.api.organization_alerts(
                "foo",
                secret_types=[
                    "google_api_key",
                    "hashicorp_vault_service_token",
                ],
            )
        )
        alert = await anext(async_it)
        self.assertEqual(alert.number, 2)
        alert = await anext(async_it)
        self.assertEqual(alert.number, 1)

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/orgs/foo/secret-scanning/alerts",
            params={
                "per_page": "100",
                "sort": "created",
                "direction": "desc",
                "secret_type": "google_api_key,hashicorp_vault_service_token",
            },
        )

    async def test_organization_alerts_resolutions(self):
        response = create_response()
        response.json.return_value = ALERTS

        self.client.get_all.return_value = AsyncIteratorMock([response])

        async_it = aiter(
            self.api.organization_alerts(
                "foo",
                resolutions=["false_positive", "wont_fix", "revoked"],
            )
        )
        alert = await anext(async_it)
        self.assertEqual(alert.number, 2)
        alert = await anext(async_it)
        self.assertEqual(alert.number, 1)

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/orgs/foo/secret-scanning/alerts",
            params={
                "per_page": "100",
                "sort": "created",
                "direction": "desc",
                "resolution": "false_positive,wont_fix,revoked",
            },
        )

    async def test_organization_alerts_sort(self):
        response = create_response()
        response.json.return_value = ALERTS

        self.client.get_all.return_value = AsyncIteratorMock([response])

        async_it = aiter(
            self.api.organization_alerts("foo", sort=AlertSort.UPDATED)
        )
        alert = await anext(async_it)
        self.assertEqual(alert.number, 2)
        alert = await anext(async_it)
        self.assertEqual(alert.number, 1)

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/orgs/foo/secret-scanning/alerts",
            params={
                "per_page": "100",
                "sort": "updated",
                "direction": "desc",
            },
        )

    async def test_organization_alerts_direction(self):
        response = create_response()
        response.json.return_value = ALERTS

        self.client.get_all.return_value = AsyncIteratorMock([response])

        async_it = aiter(
            self.api.organization_alerts("foo", direction=SortOrder.ASC)
        )
        alert = await anext(async_it)
        self.assertEqual(alert.number, 2)
        alert = await anext(async_it)
        self.assertEqual(alert.number, 1)

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/orgs/foo/secret-scanning/alerts",
            params={
                "per_page": "100",
                "sort": "created",
                "direction": "asc",
            },
        )

    async def test_alerts(self):
        response = create_response()
        response.json.return_value = ALERTS

        self.client.get_all.return_value = AsyncIteratorMock([response])

        async_it = aiter(self.api.alerts("foo/bar"))
        alert = await anext(async_it)
        self.assertEqual(alert.number, 2)
        alert = await anext(async_it)
        self.assertEqual(alert.number, 1)

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/repos/foo/bar/secret-scanning/alerts",
            params={"per_page": "100", "sort": "created", "direction": "desc"},
        )

    async def test_alerts_state(self):
        response = create_response()
        response.json.return_value = ALERTS

        self.client.get_all.return_value = AsyncIteratorMock([response])

        async_it = aiter(self.api.alerts("foo/bar", state=AlertState.RESOLVED))
        alert = await anext(async_it)
        self.assertEqual(alert.number, 2)
        alert = await anext(async_it)
        self.assertEqual(alert.number, 1)

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/repos/foo/bar/secret-scanning/alerts",
            params={
                "per_page": "100",
                "sort": "created",
                "direction": "desc",
                "state": "resolved",
            },
        )

    async def test_alerts_secret_types(self):
        response = create_response()
        response.json.return_value = ALERTS

        self.client.get_all.return_value = AsyncIteratorMock([response])

        async_it = aiter(
            self.api.alerts(
                "foo/bar",
                secret_types=[
                    "google_api_key",
                    "hashicorp_vault_service_token",
                ],
            )
        )
        alert = await anext(async_it)
        self.assertEqual(alert.number, 2)
        alert = await anext(async_it)
        self.assertEqual(alert.number, 1)

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/repos/foo/bar/secret-scanning/alerts",
            params={
                "per_page": "100",
                "sort": "created",
                "direction": "desc",
                "secret_type": "google_api_key,hashicorp_vault_service_token",
            },
        )

    async def test_alerts_resolutions(self):
        response = create_response()
        response.json.return_value = ALERTS

        self.client.get_all.return_value = AsyncIteratorMock([response])

        async_it = aiter(
            self.api.alerts(
                "foo/bar",
                resolutions=["false_positive", "wont_fix", "revoked"],
            )
        )
        alert = await anext(async_it)
        self.assertEqual(alert.number, 2)
        alert = await anext(async_it)
        self.assertEqual(alert.number, 1)

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/repos/foo/bar/secret-scanning/alerts",
            params={
                "per_page": "100",
                "sort": "created",
                "direction": "desc",
                "resolution": "false_positive,wont_fix,revoked",
            },
        )

    async def test_alerts_sort(self):
        response = create_response()
        response.json.return_value = ALERTS

        self.client.get_all.return_value = AsyncIteratorMock([response])

        async_it = aiter(self.api.alerts("foo/bar", sort=AlertSort.UPDATED))
        alert = await anext(async_it)
        self.assertEqual(alert.number, 2)
        alert = await anext(async_it)
        self.assertEqual(alert.number, 1)

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/repos/foo/bar/secret-scanning/alerts",
            params={
                "per_page": "100",
                "sort": "updated",
                "direction": "desc",
            },
        )

    async def test_alerts_direction(self):
        response = create_response()
        response.json.return_value = ALERTS

        self.client.get_all.return_value = AsyncIteratorMock([response])

        async_it = aiter(self.api.alerts("foo/bar", direction=SortOrder.ASC))
        alert = await anext(async_it)
        self.assertEqual(alert.number, 2)
        alert = await anext(async_it)
        self.assertEqual(alert.number, 1)

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/repos/foo/bar/secret-scanning/alerts",
            params={
                "per_page": "100",
                "sort": "created",
                "direction": "asc",
            },
        )

    async def test_alert(self):
        response = create_response()
        response.json.return_value = {
            "number": 42,
            "created_at": "2020-11-06T18:18:30Z",
            "url": "https://api.github.com/repos/owner/private-repo/secret-scanning/alerts/42",
            "html_url": "https://github.com/owner/private-repo/security/secret-scanning/42",
            "locations_url": "https://api.github.com/repos/owner/private-repo/secret-scanning/alerts/42/locations",
            "state": "open",
            "resolution": None,
            "resolved_at": None,
            "resolved_by": None,
            "secret_type": "mailchimp_api_key",
            "secret_type_display_name": "Mailchimp API Key",
            "secret": "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX-us2",
            "push_protection_bypassed_by": None,
            "push_protection_bypassed": False,
            "push_protection_bypassed_at": None,
            "resolution_comment": None,
        }
        self.client.get.return_value = response

        alert = await self.api.alert(
            "foo/bar",
            42,
        )

        self.client.get.assert_awaited_once_with(
            "/repos/foo/bar/secret-scanning/alerts/42",
        )

        self.assertEqual(alert.number, 42)

    async def test_update(self):
        response = create_response()
        response.json.return_value = {
            "number": 42,
            "created_at": "2020-11-06T18:18:30Z",
            "url": "https://api.github.com/repos/owner/private-repo/secret-scanning/alerts/42",
            "html_url": "https://github.com/owner/private-repo/security/secret-scanning/42",
            "locations_url": "https://api.github.com/repos/owner/private-repo/secret-scanning/alerts/42/locations",
            "state": "resolved",
            "resolution": "used_in_tests",
            "resolved_at": "2020-11-16T22:42:07Z",
            "resolved_by": {
                "login": "monalisa",
                "id": 2,
                "node_id": "MDQ6VXNlcjI=",
                "avatar_url": "https://alambic.github.com/avatars/u/2?",
                "gravatar_id": "",
                "url": "https://api.github.com/users/monalisa",
                "html_url": "https://github.com/monalisa",
                "followers_url": "https://api.github.com/users/monalisa/followers",
                "following_url": "https://api.github.com/users/monalisa/following{/other_user}",
                "gists_url": "https://api.github.com/users/monalisa/gists{/gist_id}",
                "starred_url": "https://api.github.com/users/monalisa/starred{/owner}{/repo}",
                "subscriptions_url": "https://api.github.com/users/monalisa/subscriptions",
                "organizations_url": "https://api.github.com/users/monalisa/orgs",
                "repos_url": "https://api.github.com/users/monalisa/repos",
                "events_url": "https://api.github.com/users/monalisa/events{/privacy}",
                "received_events_url": "https://api.github.com/users/monalisa/received_events",
                "type": "User",
                "site_admin": True,
            },
            "secret_type": "mailchimp_api_key",
            "secret_type_display_name": "Mailchimp API Key",
            "secret": "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX-us2",
            "push_protection_bypassed": False,
            "push_protection_bypassed_by": None,
            "push_protection_bypassed_at": None,
            "resolution_comment": "Example comment",
        }
        self.client.patch.return_value = response

        alert = await self.api.update_alert(
            "foo/bar",
            1,
            AlertState.RESOLVED,
            resolution=Resolution.USED_IN_TESTS,
            resolution_comment="Only used in tests",
        )

        self.client.patch.assert_awaited_once_with(
            "/repos/foo/bar/secret-scanning/alerts/1",
            data={
                "state": "resolved",
                "resolution": "used_in_tests",
                "resolution_comment": "Only used in tests",
            },
        )

        self.assertEqual(alert.number, 42)
        self.assertIsNone(alert.repository)

    async def test_alerts_locations(self):
        response = create_response()
        response.json.return_value = [
            {
                "type": "commit",
                "details": {
                    "path": "/example/secrets.txt",
                    "start_line": 1,
                    "end_line": 1,
                    "start_column": 1,
                    "end_column": 64,
                    "blob_sha": "af5626b4a114abcb82d63db7c8082c3c4756e51b",
                    "blob_url": "https://api.github.com/repos/octocat/hello-world/git/blobs/af5626b4a114abcb82d63db7c8082c3c4756e51b",
                    "commit_sha": "f14d7debf9775f957cf4f1e8176da0786431f72b",
                    "commit_url": "https://api.github.com/repos/octocat/hello-world/git/commits/f14d7debf9775f957cf4f1e8176da0786431f72b",
                },
            },
            {
                "type": "issue_title",
                "details": {
                    "issue_title_url": "https://api.github.com/repos/octocat/Hello-World/issues/1347"
                },
            },
            {
                "type": "issue_body",
                "details": {
                    "issue_body_url": "https://api.github.com/repos/octocat/Hello-World/issues/1347"
                },
            },
            {
                "type": "issue_comment",
                "details": {
                    "issue_comment_url": "https://api.github.com/repos/octocat/Hello-World/issues/comments/1081119451"
                },
            },
        ]

        self.client.get_all.return_value = AsyncIteratorMock([response])

        async_it = aiter(self.api.locations("foo/bar", 123))
        location = await anext(async_it)
        self.assertEqual(location.type, LocationType.COMMIT)
        location = await anext(async_it)
        self.assertEqual(location.type, LocationType.ISSUE_TITLE)
        location = await anext(async_it)
        self.assertEqual(location.type, LocationType.ISSUE_BODY)
        location = await anext(async_it)
        self.assertEqual(location.type, LocationType.ISSUE_COMMENT)

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/repos/foo/bar/secret-scanning/alerts/123/locations",
            params={
                "per_page": "100",
            },
        )



# SPDX-FileCopyrightText: 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

from dataclasses import dataclass
from datetime import datetime
from typing import Optional

from pontos.github.models.base import GitHubModel, User
from pontos.github.models.organization import Repository
from pontos.models import StrEnum


class AlertSort(StrEnum):
    """
    The property by which to sort the alerts
    """

    CREATED = "created"
    UPDATED = "updated"


class AlertState(StrEnum):
    """
    State of the GitHub Dependabot Security Alert
    """

    AUTO_DISMISSED = "auto_dismissed"
    DISMISSED = "dismissed"
    FIXED = "fixed"
    OPEN = "open"


class DismissedReason(StrEnum):
    """
    Reason phrase for a dismissed Dependabot alert
    """

    FIX_STARTED = "fix_started"
    INACCURATE = "inaccurate"
    NO_BANDWIDTH = "no_bandwidth"
    NOT_USED = "not_used"
    TOLERABLE_RISK = "tolerable_risk"


class DependencyScope(StrEnum):
    """
    The execution scope of the vulnerable dependency
    """

    DEVELOPMENT = "development"
    RUNTIME = "runtime"


class Severity(StrEnum):
    """
    The severity of the vulnerability
    """

    LOW = "low"
    MEDIUM = "medium"
    HIGH = "high"
    CRITICAL = "critical"


class IdentifierType(StrEnum):
    """
    The type of advisory identifier
    """

    CVE = "CVE"
    GHSA = "GHSA"


@dataclass
class VulnerablePackage(GitHubModel):
    """
    Details about a vulnerable Package

    Attributes:
        ecosystem: The package's language or package management ecosystem
        name: The unique package name within its ecosystem
    """

    ecosystem: str
    name: str


@dataclass
class PatchedVersion(GitHubModel):
    """
    Details pertaining to the package version that patches a vulnerability

    Attributes:
        identifier: The package version that patches the vulnerability
    """

    identifier: str


@dataclass
class Vulnerability(GitHubModel):
    """
    Details pertaining to one vulnerable version range for the advisory

    Attributes:
        package: Details about the vulnerable package
        severity: The severity of the vulnerability
        vulnerable_version_range: Conditions that identify vulnerable versions
            of this vulnerability's package
        first_patched_version: The package version that patches this
            vulnerability
    """

    package: VulnerablePackage
    severity: Severity
    vulnerable_version_range: str
    first_patched_version: Optional[PatchedVersion] = None


@dataclass
class Dependency(GitHubModel):
    """
    Details for the vulnerable dependency

    Attributes:
        package: Details about the vulnerable package
        manifest_path: The full path to the dependency manifest file, relative
            to the root of the repository
        scope: The execution scope of the vulnerable dependency
    """

    package: VulnerablePackage
    manifest_path: str
    scope: Optional[DependencyScope] = None


@dataclass
class CVSS(GitHubModel):
    """
    Details for the advisory pertaining to the Common Vulnerability Scoring
    System (CVSS)

    Attributes:
        score: The overall CVSS score of the advisory
        vector_string: The full CVSS vector string for the advisory
    """

    score: float
    vector_string: Optional[str] = None


@dataclass
class CWE(GitHubModel):
    """
    Details for the advisory pertaining to Common Weakness Enumeration (CWE)

    Attributes:
        cwe_id: The unique CWE ID
        name: The short, plain text name of the CWE
    """

    cwe_id: str
    name: str


@dataclass
class Identifier(GitHubModel):
    """
    An advisory identifier

    Attributes:
        type: The type of advisory identifier
        value: The value of the advisory identifier
    """

    type: IdentifierType
    value: str


@dataclass
class Reference(GitHubModel):
    """
    A link to additional advisory information

    Attributes:
        url: URL for the additional advisory information
    """

    url: str


@dataclass
class SecurityAdvisory(GitHubModel):
    """
    Details for the GitHub Security Advisory

    Attributes:
        ghsa_id: The unique GitHub Security Advisory ID assigned to the advisory
        cve_id: The unique CVE ID assigned to the advisory
        summary: A short, plain text summary of the advisory
        description: A long-form Markdown-supported description of the advisory
        vulnerabilities: Vulnerable version range information for the advisory
        severity: The severity of the advisory
        cvss: The overall CVSS score of the advisory
        cwes: CWE weaknesses assigned to the advisory
        identifiers: Values that identify this advisory among security
            information sources
        references: Links to additional advisory information
        published_at: The time that the advisory was published
        updated_at: The time that the advisory was last modified
        withdrawn_at: The time that the advisory was withdrawn
    """

    ghsa_id: str
    summary: str
    description: str
    vulnerabilities: list[Vulnerability]
    severity: Severity
    cvss: CVSS
    cwes: list[CWE]
    identifiers: list[Identifier]
    references: list[Reference]
    published_at: datetime
    updated_at: datetime
    cve_id: Optional[str] = None
    withdrawn_at: Optional[datetime] = None


@dataclass
class DependabotAlert(GitHubModel):
    """
    A GitHub dependabot security alert model

    Attributes:
        number: The security alert number
        state: The state of the Dependabot alert
        dependency: Details for the vulnerable dependency
        security_advisory: Details for the GitHub Security Advisory
        security_vulnerability: Details pertaining to one vulnerable version
            range for the advisory
        url: The REST API URL of the alert resource
        html_url: The GitHub URL of the alert resource
        created_at: The time that the alert was created
        updated_at: The time that the alert was last updated
        dismissed_at: The time that the alert was dismissed
        dismissed_by: User who dismissed the alert
        dismissed_reason: The reason that the alert was dismissed
        dismissed_comment: An optional comment associated with the alert's
            dismissal
        fixed_at: The time that the alert was no longer detected and was
            considered fixed
        auto_dismissed_at: The time that the alert was auto-dismissed
        repository: The GitHub repository containing the alert. It's not
            returned when requesting a specific alert
    """

    number: int
    state: AlertState
    dependency: Dependency
    security_advisory: SecurityAdvisory
    security_vulnerability: Vulnerability
    url: str
    html_url: str
    created_at: datetime
    updated_at: datetime
    repository: Optional[Repository] = None
    dismissed_at: Optional[datetime] = None
    dismissed_by: Optional[User] = None
    dismissed_reason: Optional[DismissedReason] = None
    dismissed_comment: Optional[str] = None
    fixed_at: Optional[datetime] = None
    auto_dismissed_at: Optional[datetime] = None



# SPDX-FileCopyrightText: 2022 - 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

# ruff: noqa: E501

import unittest

from pontos.cpe import ANY, CPE, NA, CPEParsingError, Part
from pontos.cpe._cpe import (
    bind_value_for_formatted_string,
    convert_double_backslash,
    split_cpe,
    unbind_value_from_formatted_string,
    unquote_attribute_value,
)


class SplitCpeTestCase(unittest.TestCase):
    def test_split_uri_cpe(self):
        parts = split_cpe("cpe:/o:microsoft:windows_xp:::pro")

        self.assertEqual(len(parts), 7)
        self.assertEqual(parts[0], "cpe")
        self.assertEqual(parts[1], "/o")
        self.assertEqual(parts[2], "microsoft")
        self.assertEqual(parts[3], "windows_xp")
        self.assertEqual(parts[4], "")
        self.assertEqual(parts[5], "")
        self.assertEqual(parts[6], "pro")

    def test_split_formatted_cpe(self):
        parts = split_cpe(
            "cpe:2.3:a:microsoft:internet_explorer:8.0.6001:beta:*:*:*:*:*:*"
        )

        self.assertEqual(len(parts), 13)
        self.assertEqual(parts[0], "cpe")
        self.assertEqual(parts[1], "2.3")
        self.assertEqual(parts[2], "a")
        self.assertEqual(parts[3], "microsoft")
        self.assertEqual(parts[4], "internet_explorer")
        self.assertEqual(parts[5], "8.0.6001")
        self.assertEqual(parts[6], "beta")
        self.assertEqual(parts[7], "*")
        self.assertEqual(parts[8], "*")
        self.assertEqual(parts[9], "*")
        self.assertEqual(parts[10], "*")
        self.assertEqual(parts[11], "*")
        self.assertEqual(parts[12], "*")

        parts = split_cpe("cpe:2.3:a:foo:bar\:mumble:1.0:*:*:*:*:*:*:*")

        self.assertEqual(len(parts), 13)
        self.assertEqual(parts[0], "cpe")
        self.assertEqual(parts[1], "2.3")
        self.assertEqual(parts[2], "a")
        self.assertEqual(parts[3], "foo")
        self.assertEqual(parts[4], "bar\\:mumble")
        self.assertEqual(parts[5], "1.0")
        self.assertEqual(parts[6], "*")
        self.assertEqual(parts[7], "*")
        self.assertEqual(parts[8], "*")
        self.assertEqual(parts[9], "*")
        self.assertEqual(parts[10], "*")
        self.assertEqual(parts[11], "*")
        self.assertEqual(parts[12], "*")


class ConvertDoubleBackslashTestCase(unittest.TestCase):
    def test_remove_backslash(self):
        self.assertEqual(convert_double_backslash("foo-bar"), "foo-bar")
        self.assertEqual(convert_double_backslash("foo\\bar"), "foo\\bar")
        self.assertEqual(convert_double_backslash("foo\\\\bar"), "foo\\bar")


class UnbindValueFromFormattedStringTestCase(unittest.TestCase):
    def test_unchanged(self):
        self.assertIsNone(unbind_value_from_formatted_string(None))
        self.assertEqual(unbind_value_from_formatted_string(ANY), ANY)
        self.assertEqual(unbind_value_from_formatted_string(NA), NA)

        self.assertEqual(
            unbind_value_from_formatted_string("foo_bar"), "foo_bar"
        )
        self.assertEqual(
            unbind_value_from_formatted_string("foo\\:bar"), "foo\\:bar"
        )
        self.assertEqual(
            unbind_value_from_formatted_string("1\\.2\\.3"), "1\\.2\\.3"
        )

    def test_quoting(self):
        self.assertEqual(
            unbind_value_from_formatted_string("foo-bar"), "foo\\-bar"
        )
        self.assertEqual(  # not sure if this can happen and if it's valid
            unbind_value_from_formatted_string("foo:bar"), "foo\\:bar"
        )
        self.assertEqual(
            unbind_value_from_formatted_string("1.2.3"), "1\\.2\\.3"
        )

    def test_asterisk(self):
        self.assertEqual(unbind_value_from_formatted_string("*foo"), "*foo")
        self.assertEqual(unbind_value_from_formatted_string("foo*"), "foo*")
        self.assertEqual(unbind_value_from_formatted_string("foo\\*"), "foo\\*")

        with self.assertRaisesRegex(
            CPEParsingError,
            "An unquoted asterisk must appear at the beginning or end of "
            "'foo\*bar'",
        ):
            unbind_value_from_formatted_string("foo*bar")

        with self.assertRaisesRegex(
            CPEParsingError,
            "An unquoted asterisk must appear at the beginning or end of "
            "'\*\*foo'",
        ):
            unbind_value_from_formatted_string("**foo")

    def test_question_mark(self):
        self.assertEqual(unbind_value_from_formatted_string("?foo"), "?foo")
        self.assertEqual(unbind_value_from_formatted_string("??foo"), "??foo")
        self.assertEqual(unbind_value_from_formatted_string("foo?"), "foo?")
        self.assertEqual(unbind_value_from_formatted_string("foo??"), "foo??")
        self.assertEqual(unbind_value_from_formatted_string("foo\\?"), "foo\\?")

        with self.assertRaisesRegex(
            CPEParsingError,
            "An unquoted question mark must appear at the beginning or end, "
            "or in a leading or trailing sequence 'foo\?bar'",
        ):
            unbind_value_from_formatted_string("foo?bar")


class BindValueForFormattedStringTestCase(unittest.TestCase):
    def test_any(self):
        self.assertEqual(bind_value_for_formatted_string(None), ANY)
        self.assertEqual(bind_value_for_formatted_string(""), ANY)
        self.assertEqual(bind_value_for_formatted_string(ANY), ANY)

    def test_na(self):
        self.assertEqual(bind_value_for_formatted_string(NA), NA)

    def test_remove_quoting(self):
        self.assertEqual(bind_value_for_formatted_string("1\\.2\\.3"), "1.2.3")
        # _ doesn't get quoted during unbinding therefore unquoting it here
        # doesn't really make sense bit it's in the standard!
        self.assertEqual(
            bind_value_for_formatted_string("foo\\_bar"), "foo_bar"
        )
        self.assertEqual(
            bind_value_for_formatted_string("foo\\-bar"), "foo-bar"
        )

    def test_unchanged(self):
        self.assertEqual(
            bind_value_for_formatted_string("foo\\:bar"), "foo\\:bar"
        )
        self.assertEqual(bind_value_for_formatted_string("?foo"), "?foo")
        self.assertEqual(bind_value_for_formatted_string("foo*"), "foo*")
        self.assertEqual(bind_value_for_formatted_string("foo\\*"), "foo\\*")


class UnquoteAttributeValueTestCase(unittest.TestCase):
    def test_unchanged(self):
        self.assertIsNone(unquote_attribute_value(None))
        self.assertEqual(unquote_attribute_value(""), "")
        self.assertEqual(unquote_attribute_value(ANY), ANY)
        self.assertEqual(unquote_attribute_value("?"), "?")
        self.assertEqual(unquote_attribute_value("foo-bar"), "foo-bar")
        self.assertEqual(unquote_attribute_value("foo_bar"), "foo_bar")
        self.assertEqual(unquote_attribute_value("1.2.3"), "1.2.3")

    def test_special(self):
        self.assertEqual(unquote_attribute_value("foo\\?bar"), "foo\\?bar")
        self.assertEqual(unquote_attribute_value("foo\\*bar"), "foo\\*bar")

    def test_unquote(self):
        self.assertEqual(unquote_attribute_value("foo\\\\bar"), "foo\\bar")
        self.assertEqual(unquote_attribute_value("foo\\:bar"), "foo:bar")
        self.assertEqual(unquote_attribute_value("1\\.2\\.3"), "1.2.3")


class CPETestCase(unittest.TestCase):
    def test_uri_binding(self):
        cpe_string = "cpe:/o:microsoft:windows_xp:::pro"
        cpe = CPE.from_string(cpe_string)

        self.assertEqual(str(cpe), "cpe:/o:microsoft:windows_xp:::pro")
        self.assertEqual(
            cpe.as_uri_binding(), "cpe:/o:microsoft:windows_xp:::pro"
        )
        self.assertEqual(
            cpe.as_formatted_string_binding(),
            "cpe:2.3:o:microsoft:windows_xp:*:*:pro:*:*:*:*:*",
        )
        self.assertTrue(cpe.is_uri_binding())
        self.assertFalse(cpe.is_formatted_string_binding())
        self.assertEqual(cpe.part, Part.OPERATING_SYSTEM)
        self.assertEqual(cpe.vendor, "microsoft")
        self.assertEqual(cpe.product, "windows_xp")
        self.assertEqual(cpe.edition, "pro")
        self.assertEqual(cpe.version, ANY)
        self.assertEqual(cpe.update, ANY)
        self.assertIsNone(cpe.language)
        self.assertIsNone(cpe.sw_edition)
        self.assertIsNone(cpe.target_sw)
        self.assertIsNone(cpe.target_hw)
        self.assertIsNone(cpe.other)

        cpe = CPE.from_string(
            "cpe:/a:foo%5cbar:big%24money_manager_2010:::~~special~ipod_touch~80gb~"
        )
        self.assertEqual(
            str(cpe),
            "cpe:/a:foo%5cbar:big%24money_manager_2010:::~~special~ipod_touch~80gb~",
        )
        self.assertEqual(
            cpe.as_uri_binding(),
            "cpe:/a:foo%5cbar:big%24money_manager_2010:::~~special~ipod_touch~80gb~",
        )
        self.assertEqual(
            cpe.as_formatted_string_binding(),
            "cpe:2.3:a:foo\\\\bar:big\$money_manager_2010:*:*:*:*:special:ipod_touch:80gb:*",
        )
        self.assertTrue(cpe.is_uri_binding())
        self.assertFalse(cpe.is_formatted_string_binding())
        self.assertEqual(cpe.part, Part.APPLICATION)
        self.assertEqual(cpe.vendor, "foo\\bar")
        self.assertEqual(cpe.product, "big$money_manager_2010")
        self.assertEqual(cpe.version, ANY)
        self.assertEqual(cpe.update, ANY)
        self.assertIsNone(cpe.language)
        self.assertIsNone(cpe.edition)
        self.assertEqual(cpe.sw_edition, "special")
        self.assertEqual(cpe.target_sw, "ipod_touch")
        self.assertEqual(cpe.target_hw, "80gb")
        self.assertIsNone(cpe.other)

    def test_formatted_string_binding(self):
        cpe_string = (
            "cpe:2.3:a:qrokes:qr_twitter_widget:*:*:*:*:*:wordpress:*:*"
        )
        cpe = CPE.from_string(cpe_string)

        self.assertEqual(
            str(cpe),
            "cpe:2.3:a:qrokes:qr_twitter_widget:*:*:*:*:*:wordpress:*:*",
        )
        self.assertEqual(
            cpe.as_uri_binding(),
            "cpe:/a:qrokes:qr_twitter_widget:::~~~wordpress~~",
        )
        self.assertEqual(
            cpe.as_formatted_string_binding(),
            "cpe:2.3:a:qrokes:qr_twitter_widget:*:*:*:*:*:wordpress:*:*",
        )
        self.assertFalse(cpe.is_uri_binding())
        self.assertTrue(cpe.is_formatted_string_binding())
        self.assertEqual(cpe.part, Part.APPLICATION)
        self.assertEqual(cpe.vendor, "qrokes")
        self.assertEqual(cpe.product, "qr_twitter_widget")
        self.assertEqual(cpe.version, ANY)
        self.assertEqual(cpe.update, ANY)
        self.assertEqual(cpe.edition, ANY)
        self.assertEqual(cpe.language, ANY)
        self.assertEqual(cpe.sw_edition, ANY)
        self.assertEqual(cpe.target_sw, "wordpress")
        self.assertEqual(cpe.target_hw, ANY)
        self.assertEqual(cpe.other, ANY)

    def test_uri_bind_examples(self):
        # test examples from https://nvlpubs.nist.gov/nistpubs/Legacy/IR/nistir7695.pdf

        # example 1
        cpe = CPE(
            part=Part.APPLICATION,
            vendor="microsoft",
            product="internet_explorer",
            version="8\.0\.6001",
            update="beta",
            edition=ANY,
        )
        self.assertEqual(
            cpe.as_uri_binding(),
            "cpe:/a:microsoft:internet_explorer:8.0.6001:beta",
        )

        # example 2
        cpe = CPE(
            part=Part.APPLICATION,
            vendor="microsoft",
            product="internet_explorer",
            version="8\.*",
            update="sp?",
        )
        self.assertEqual(
            cpe.as_uri_binding(),
            "cpe:/a:microsoft:internet_explorer:8.%02:sp%01",
        )

        # example 3
        cpe = CPE(
            part=Part.APPLICATION,
            vendor="hp",
            product="insight_diagnostics",
            version="7\.4\.0\.1570",
            update=NA,
            sw_edition="online",
            target_sw="win2003",
            target_hw="x64",
        )
        self.assertEqual(
            cpe.as_uri_binding(),
            "cpe:/a:hp:insight_diagnostics:7.4.0.1570:-:~~online~win2003~x64~",
        )

        # example 4
        cpe = CPE(
            part=Part.APPLICATION,
            vendor="hp",
            product="openview_network_manager",
            version="7\.51",
            target_sw="linux",
        )
        self.assertEqual(
            cpe.as_uri_binding(),
            "cpe:/a:hp:openview_network_manager:7.51::~~~linux~~",
        )

        # example 5
        cpe = CPE(
            part=Part.APPLICATION,
            vendor="foo\\\\bar",
            product="big\$money_manager_2010",
            sw_edition="special",
            target_sw="ipod_touch",
            target_hw="80gb",
        )
        self.assertEqual(
            cpe.as_uri_binding(),
            "cpe:/a:foo%5cbar:big%24money_manager_2010:::~~special~ipod_touch~80gb~",
        )

    def test_uri_unbind_examples(self):
        # test examples from https://nvlpubs.nist.gov/nistpubs/Legacy/IR/nistir7695.pdf

        # example 1
        cpe = CPE.from_string(
            "cpe:/a:microsoft:internet_explorer:8.0.6001:beta"
        )
        self.assertTrue(cpe.is_uri_binding())
        self.assertFalse(cpe.is_formatted_string_binding())
        self.assertEqual(cpe.part, Part.APPLICATION)
        self.assertEqual(cpe.vendor, "microsoft")
        self.assertEqual(cpe.product, "internet_explorer")
        self.assertEqual(cpe.version, "8.0.6001")
        self.assertEqual(cpe.update, "beta")
        self.assertIsNone(cpe.language)
        self.assertIsNone(cpe.edition)
        self.assertIsNone(cpe.sw_edition)
        self.assertIsNone(cpe.target_sw)
        self.assertIsNone(cpe.target_hw)
        self.assertIsNone(cpe.other)

        # example 2
        cpe = CPE.from_string("cpe:/a:microsoft:internet_explorer:8.%2a:sp%3f")
        self.assertTrue(cpe.is_uri_binding())
        self.assertFalse(cpe.is_formatted_string_binding())
        self.assertEqual(cpe.part, Part.APPLICATION)
        self.assertEqual(cpe.vendor, "microsoft")
        self.assertEqual(cpe.product, "internet_explorer")
        self.assertEqual(cpe.version, "8.\\*")
        self.assertEqual(cpe.update, "sp\\?")
        self.assertIsNone(cpe.language)
        self.assertIsNone(cpe.edition)
        self.assertIsNone(cpe.sw_edition)
        self.assertIsNone(cpe.target_sw)
        self.assertIsNone(cpe.target_hw)
        self.assertIsNone(cpe.other)

        # example 3
        cpe = CPE.from_string("cpe:/a:microsoft:internet_explorer:8.%02:sp%01")
        self.assertTrue(cpe.is_uri_binding())
        self.assertFalse(cpe.is_formatted_string_binding())
        self.assertEqual(cpe.part, Part.APPLICATION)
        self.assertEqual(cpe.vendor, "microsoft")
        self.assertEqual(cpe.product, "internet_explorer")
        self.assertEqual(cpe.version, "8.*")
        self.assertEqual(cpe.update, "sp?")
        self.assertIsNone(cpe.language)
        self.assertIsNone(cpe.edition)
        self.assertIsNone(cpe.sw_edition)
        self.assertIsNone(cpe.target_sw)
        self.assertIsNone(cpe.target_hw)
        self.assertIsNone(cpe.other)

        # example 4
        cpe = CPE.from_string(
            "cpe:/a:hp:insight_diagnostics:7.4.0.1570::~~online~win2003~x64~"
        )
        self.assertTrue(cpe.is_uri_binding())
        self.assertFalse(cpe.is_formatted_string_binding())
        self.assertEqual(cpe.part, Part.APPLICATION)
        self.assertEqual(cpe.vendor, "hp")
        self.assertEqual(cpe.product, "insight_diagnostics")
        self.assertEqual(cpe.version, "7.4.0.1570")
        self.assertEqual(cpe.update, ANY)
        self.assertIsNone(cpe.language)
        self.assertIsNone(cpe.edition)
        self.assertEqual(cpe.sw_edition, "online")
        self.assertEqual(cpe.target_sw, "win2003")
        self.assertEqual(cpe.target_hw, "x64")
        self.assertIsNone(cpe.other)

        # example 5
        cpe = CPE.from_string(
            "cpe:/a:hp:openview_network_manager:7.51:-:~~~linux~~"
        )
        self.assertTrue(cpe.is_uri_binding())
        self.assertFalse(cpe.is_formatted_string_binding())
        self.assertEqual(cpe.part, Part.APPLICATION)
        self.assertEqual(cpe.vendor, "hp")
        self.assertEqual(cpe.product, "openview_network_manager")
        self.assertEqual(cpe.version, "7.51")
        self.assertEqual(cpe.update, NA)
        self.assertIsNone(cpe.language)
        self.assertIsNone(cpe.edition)
        self.assertIsNone(cpe.sw_edition)
        self.assertEqual(cpe.target_sw, "linux")
        self.assertIsNone(cpe.target_hw)
        self.assertIsNone(cpe.other)

        # example 6
        # with self.assertRaises(CPEParsingError):
        #     CPE.from_string(
        #         "cpe:/a:foo%5cbar:big%24money_2010%07:::~~special~ipod_touch~80gb~"
        #     )

        # example 7
        cpe = CPE.from_string("cpe:/a:foo~bar:big%7emoney_2010")
        self.assertTrue(cpe.is_uri_binding())
        self.assertFalse(cpe.is_formatted_string_binding())
        self.assertEqual(cpe.part, Part.APPLICATION)
        self.assertEqual(cpe.vendor, "foo~bar")
        self.assertEqual(cpe.product, "big~money_2010")
        self.assertIsNone(cpe.version)
        self.assertIsNone(cpe.update)
        self.assertIsNone(cpe.language)
        self.assertIsNone(cpe.edition)
        self.assertIsNone(cpe.sw_edition)
        self.assertIsNone(cpe.target_sw)
        self.assertIsNone(cpe.target_hw)
        self.assertIsNone(cpe.other)

        # example 8
        with self.assertRaisesRegex(
            CPEParsingError,
            "^Percent-encoded asterisk is no at the beginning or the end "
            "of '12.%02.1234'$",
        ):
            CPE.from_string("cpe:/a:foo:bar:12.%02.1234")

    def test_formatted_bind_examples(self):
        # test examples from https://nvlpubs.nist.gov/nistpubs/Legacy/IR/nistir7695.pdf

        # example 1
        cpe = CPE(
            part=Part.APPLICATION,
            vendor="microsoft",
            product="internet_explorer",
            version="8\.0\.6001",
            update="beta",
            edition=ANY,
        )
        self.assertEqual(
            cpe.as_formatted_string_binding(),
            "cpe:2.3:a:microsoft:internet_explorer:8.0.6001:beta:*:*:*:*:*:*",
        )

        # example 2
        cpe = CPE(
            part=Part.APPLICATION,
            vendor="microsoft",
            product="internet_explorer",
            version="8\.*",
            update="sp?",
            edition=ANY,
        )
        self.assertEqual(
            cpe.as_formatted_string_binding(),
            "cpe:2.3:a:microsoft:internet_explorer:8.*:sp?:*:*:*:*:*:*",
        )

        cpe = CPE(
            part=Part.APPLICATION,
            vendor="microsoft",
            product="internet_explorer",
            version="8\.\*",
            update="sp?",
        )
        self.assertEqual(
            cpe.as_formatted_string_binding(),
            "cpe:2.3:a:microsoft:internet_explorer:8.\*:sp?:*:*:*:*:*:*",
        )

        # example 3
        cpe = CPE(
            part=Part.APPLICATION,
            vendor="hp",
            product="insight",
            version="7\.4\.0\.1570",
            update=NA,
            sw_edition="online",
            target_sw="win2003",
            target_hw="x64",
        )
        self.assertEqual(
            cpe.as_formatted_string_binding(),
            "cpe:2.3:a:hp:insight:7.4.0.1570:-:*:*:online:win2003:x64:*",
        )

        # example 4
        cpe = CPE(
            part=Part.APPLICATION,
            vendor="hp",
            product="openview_network_manager",
            version="7\.51",
            target_sw="linux",
        )
        self.assertEqual(
            cpe.as_formatted_string_binding(),
            "cpe:2.3:a:hp:openview_network_manager:7.51:*:*:*:*:linux:*:*",
        )

        # example 5
        cpe = CPE(
            part=Part.APPLICATION,
            vendor="foo\\\\bar",
            product="big\$money_2010",
            sw_edition="special",
            target_sw="ipod_touch",
            target_hw="80gb",
        )
        self.assertEqual(
            cpe.as_formatted_string_binding(),
            "cpe:2.3:a:foo\\\\bar:big\$money_2010:*:*:*:*:special:ipod_touch:80gb:*",
        )

    def test_formatted_unbind_examples(self):
        # test examples from https://nvlpubs.nist.gov/nistpubs/Legacy/IR/nistir7695.pdf

        # example 1
        cpe = CPE.from_string(
            "cpe:2.3:a:microsoft:internet_explorer:8.0.6001:beta:*:*:*:*:*:*"
        )
        self.assertFalse(cpe.is_uri_binding())
        self.assertTrue(cpe.is_formatted_string_binding())
        self.assertEqual(cpe.part, Part.APPLICATION)
        self.assertEqual(cpe.vendor, "microsoft")
        self.assertEqual(cpe.product, "internet_explorer")
        self.assertEqual(cpe.version, "8.0.6001")
        self.assertEqual(cpe.update, "beta")
        self.assertEqual(cpe.edition, ANY)
        self.assertEqual(cpe.language, ANY)
        self.assertEqual(cpe.sw_edition, ANY)
        self.assertEqual(cpe.target_sw, ANY)
        self.assertEqual(cpe.target_hw, ANY)
        self.assertEqual(cpe.other, ANY)

        # example 2
        cpe = CPE.from_string(
            "cpe:2.3:a:microsoft:internet_explorer:8.*:sp?:*:*:*:*:*:*"
        )
        self.assertFalse(cpe.is_uri_binding())
        self.assertTrue(cpe.is_formatted_string_binding())
        self.assertEqual(cpe.part, Part.APPLICATION)
        self.assertEqual(cpe.vendor, "microsoft")
        self.assertEqual(cpe.product, "internet_explorer")
        self.assertEqual(cpe.version, "8.*")
        self.assertEqual(cpe.update, "sp?")
        self.assertEqual(cpe.language, ANY)
        self.assertEqual(cpe.edition, ANY)
        self.assertEqual(cpe.sw_edition, ANY)
        self.assertEqual(cpe.target_sw, ANY)
        self.assertEqual(cpe.target_hw, ANY)
        self.assertEqual(cpe.other, ANY)

        # example 3
        cpe = CPE.from_string(
            "cpe:2.3:a:hp:insight_diagnostics:7.4.0.1570:-:*:*:online:win2003:x64:*"
        )
        self.assertFalse(cpe.is_uri_binding())
        self.assertTrue(cpe.is_formatted_string_binding())
        self.assertEqual(cpe.part, Part.APPLICATION)
        self.assertEqual(cpe.vendor, "hp")
        self.assertEqual(cpe.product, "insight_diagnostics")
        self.assertEqual(cpe.version, "7.4.0.1570")
        self.assertEqual(cpe.update, NA)
        self.assertEqual(cpe.language, ANY)
        self.assertEqual(cpe.edition, ANY)
        self.assertEqual(cpe.sw_edition, "online")
        self.assertEqual(cpe.target_sw, "win2003")
        self.assertEqual(cpe.target_hw, "x64")
        self.assertEqual(cpe.other, ANY)

        with self.assertRaisesRegex(
            CPEParsingError,
            "^An unquoted asterisk must appear at the beginning or end of "
            "'7.4.*.1570'$",
        ):
            # embedded unquoted asterisk in the version attribute
            CPE.from_string(
                "cpe:2.3:a:hp:insight_diagnostics:7.4.*.1570:*:*:*:*:*:*"
            )

        # example 4
        cpe = CPE.from_string(
            "cpe:2.3:a:foo\\\\bar:big\$money:2010:*:*:*:special:ipod_touch:80gb:*"
        )
        self.assertFalse(cpe.is_uri_binding())
        self.assertTrue(cpe.is_formatted_string_binding())
        self.assertEqual(cpe.part, Part.APPLICATION)
        self.assertEqual(cpe.vendor, "foo\\bar")
        self.assertEqual(cpe.product, "big$money")
        self.assertEqual(cpe.version, "2010")
        self.assertEqual(cpe.update, ANY)
        self.assertEqual(cpe.edition, ANY)
        self.assertEqual(cpe.language, ANY)
        self.assertEqual(cpe.sw_edition, "special")
        self.assertEqual(cpe.target_sw, "ipod_touch")
        self.assertEqual(cpe.target_hw, "80gb")
        self.assertEqual(cpe.other, ANY)

        cpe = CPE.from_string("cpe:2.3:a:foo:bar\:mumble:1.0:*:*:*:*:*:*:*")
        self.assertFalse(cpe.is_uri_binding())
        self.assertTrue(cpe.is_formatted_string_binding())
        self.assertEqual(cpe.part, Part.APPLICATION)
        self.assertEqual(cpe.vendor, "foo")
        self.assertEqual(cpe.product, "bar:mumble")
        self.assertEqual(cpe.version, "1.0")
        self.assertEqual(cpe.update, ANY)
        self.assertEqual(cpe.edition, ANY)
        self.assertEqual(cpe.language, ANY)
        self.assertEqual(cpe.sw_edition, ANY)
        self.assertEqual(cpe.target_sw, ANY)
        self.assertEqual(cpe.target_hw, ANY)
        self.assertEqual(cpe.other, ANY)

    def test_as_uri_binding(self):
        cpe = CPE.from_string("cpe:2.3:a:microsoft:internet_explorer:8\\.*:sp?")
        self.assertEqual(
            cpe.as_uri_binding(),
            "cpe:/a:microsoft:internet_explorer:8.%02:sp%01",
        )

        cpe = CPE.from_string("cpe:2.3:a:cgiirc:cgi\:irc:0.5.7:*:*:*:*:*:*:*")
        self.assertEqual(
            cpe.as_uri_binding(),
            "cpe:/a:cgiirc:cgi%3airc:0.5.7",
        )

    def test_as_uri_binding_with_edition(self):
        cpe_string = "cpe:2.3:a:hp:insight_diagnostics:7.4.0.1570:-:*:*:online:win2003:x64"
        cpe = CPE.from_string(cpe_string)

        self.assertEqual(
            cpe.as_uri_binding(),
            "cpe:/a:hp:insight_diagnostics:7.4.0.1570:-:~~online~win2003~x64~",
        )

    def test_parse_error(self):
        with self.assertRaisesRegex(
            CPEParsingError,
            "^Invalid CPE string 'foo/bar'. CPE does not start with 'cpe:/' "
            "or 'cpe:2.3'$",
        ):
            CPE.from_string("foo/bar")

    def test_str(self):
        cpe = CPE(part=Part.APPLICATION, vendor="foo", product="bar")
        self.assertEqual(str(cpe), "cpe:/a:foo:bar")

        cpe = CPE(
            part=Part.APPLICATION,
            vendor="foo",
            product="bar",
            target_sw="ipsum",
        )
        self.assertEqual(str(cpe), "cpe:2.3:a:foo:bar:*:*:*:*:*:ipsum:*:*")

        cpe = CPE(
            cpe_string="cpe:2.3:a:foo:bar",
            part=Part.APPLICATION,
            vendor="foo",
            product="bar",
        )
        self.assertEqual(str(cpe), "cpe:2.3:a:foo:bar")

    def test_has_extended_attribute(self):
        cpe = CPE(part=Part.APPLICATION, vendor="foo", product="bar")
        self.assertFalse(cpe.has_extended_attribute())

        cpe = CPE(
            part=Part.APPLICATION,
            vendor="foo",
            product="bar",
            target_sw="ipsum",
        )
        self.assertTrue(cpe.has_extended_attribute())

    def test_clone(self):
        cpe = CPE.from_string(
            "cpe:2.3:a:hp:openview_network_manager:7.51:*:*:*:*:linux:*:*"
        )

        cpe2 = cpe.clone()
        self.assertIsNot(cpe, cpe2)

        cpe = CPE.from_string(
            "cpe:2.3:a:hp:openview_network_manager:7.51:*:*:*:*:linux:*:*"
        )
        cpe2 = cpe.clone(version=ANY)
        self.assertIsNot(cpe, cpe2)
        self.assertEqual(cpe.version, "7.51")
        self.assertEqual(cpe2.version, ANY)

    def test_equal(self):
        cpe1 = CPE.from_string("cpe:2.3:a:3com:3cdaemon:-:*:*:*:*:*:*:*")
        cpe2 = CPE.from_string("cpe:2.3:a:adobe:flash_player:-:*:*:*:*:*:*:*")
        cpe3 = CPE.from_string("cpe:2.3:a:3com:3cdaemon:-:*:*:*:*:*:*:*")

        self.assertNotEqual(cpe1, None)
        self.assertNotEqual(cpe1, "foo")
        self.assertNotEqual(cpe1, cpe2)
        self.assertIsNot(cpe1, cpe3)
        self.assertEqual(cpe1, cpe3)

    def test_hashable(self):
        cpe1 = CPE.from_string("cpe:2.3:a:3com:3cdaemon:-:*:*:*:*:*:*:*")
        cpe2 = CPE.from_string("cpe:2.3:a:adobe:flash_player:-:*:*:*:*:*:*:*")
        cpe3 = CPE.from_string("cpe:2.3:a:3com:3cdaemon:-:*:*:*:*:*:*:*")

        cpe_list = [cpe1, cpe2, cpe3, cpe1]
        self.assertTrue(len(cpe_list), 4)

        cpe_set = set(cpe_list)
        self.assertTrue(len(cpe_set), 2)

    def test_repr(self):
        cpe1 = CPE.from_string("cpe:2.3:a:3com:3cdaemon:-:*:*:*:*:*:*:*")

        self.assertEqual(
            repr(cpe1),
            '<CPE part="a" vendor="3com" product="3cdaemon" version="-" '
            'update="*" edition="*" language="*" sw_edition="*" target_sw="*" '
            'target_hw="*" other="*">',
        )



# Copyright (C) 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#


from abc import ABC

from pontos.models import StrEnum

from .base import SortOrder

__all__ = (
    "InDescriptionQualifier",
    "InNameQualifier",
    "InReadmeQualifier",
    "InTopicsQualifier",
    "IsPrivateQualifier",
    "IsPublicQualifier",
    "NotQualifier",
    "OrganizationQualifier",
    "Qualifier",
    "RepositoryQualifier",
    "RepositorySort",
    "SortOrder",
    "UserQualifier",
)


class RepositorySort(StrEnum):
    """
    Sort repositories by

    Attributes:
        STARS: GitHub starts
        FORKS: GitHub forks
        HELP_WANTED_ISSUES: Number of issues with help wanted label
        UPDATED: Last updated
    """

    STARS = "stars"
    FORKS = "forks"
    HELP_WANTED_ISSUES = "help-wanted-issues"
    UPDATED = "updated"


class Qualifier(ABC):
    """
    An abstract base class for search qualifiers

    Attributes:
        operator: The search operator
        term: The search term
    """

    operator: str
    term: str

    def __str__(self) -> str:
        """ """
        return f"{self.operator}:{self.term}"


class NotQualifier(Qualifier):
    """
    Qualifier for negating another qualifier

    Example:

        Exclude a repository from a search

        .. code-block:: python

            from pontos.github.models import NotQualifier, RepositoryQualifier

            qualifier = NotQualifier(RepositoryQualifier("foo/bar"))
    """

    def __init__(self, qualifier: Qualifier) -> None:
        self.qualifier = qualifier

    def __str__(self) -> str:
        return f"-{str(self.qualifier)}"


class InQualifier(Qualifier):
    operator = "in"


class InNameQualifier(InQualifier):
    """
    Qualifier for searching in repository names
    """

    term = "name"


class InDescriptionQualifier(InQualifier):
    """
    Qualifier for searching in repository descriptions
    """

    term = "description"


class InTopicsQualifier(InQualifier):
    """
    Qualifier for searching in repository topics
    """

    term = "topics"


class InReadmeQualifier(InQualifier):
    """
    Qualifier for searching in repository READMEs
    """

    term = "readme"


class RepositoryQualifier(Qualifier):
    """
    Qualifier for searching within a specific repository
    """

    operator = "repo"

    def __init__(self, repository: str) -> None:
        """
        Search within a repository

        Args:
            repository: owner/repo
        """
        self.term = repository


class OrganizationQualifier(Qualifier):
    """
    Qualifier for searching within a specific organization
    """

    operator = "org"

    def __init__(self, organization: str) -> None:
        """
        Search within an organization

        Args:
            organization: Name of the organization to search within
        """
        self.term = organization


class UserQualifier(Qualifier):
    """
    Qualifier for searching within a specific user space
    """

    operator = "user"

    def __init__(self, user: str) -> None:
        """
        Search within an user space

        Args:
            user: Name of the user
        """
        self.term = user


class IsPublicQualifier(Qualifier):
    """
    Qualifier for searching for public repositories
    """

    operator = "is"
    term = "public"


class IsPrivateQualifier(Qualifier):
    """
    Qualifier for searching for private repositories
    """

    operator = "is"
    term = "private"



# Copyright (C) 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from pontos.errors import PontosError


class ChangelogError(PontosError):
    """
    Some error has occurred during changelog handling
    """


class ChangelogBuilderError(ChangelogError):
    """
    An error while building a changelog
    """



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

from argparse import ArgumentParser, Namespace
from pathlib import Path
from typing import Optional, Sequence

import shtab

from pontos.version.schemes import (
    VERSIONING_SCHEMES,
    VersioningScheme,
    versioning_scheme_argument_type,
)


def parse_args(args: Optional[Sequence[str]] = None) -> Namespace:
    parser = ArgumentParser(
        description="Conventional commits utility. Create a changelog markdown "
        " text from conventional commits between the current and next release.",
        prog="pontos-changelog",
    )
    shtab.add_argument_to(parser)

    parser.add_argument(
        "--config",
        "-C",
        type=Path,
        help="Optional. Conventional commits config file (toml), including "
        "conventions. If not provided defaults are used.",
    ).complete = shtab.FILE  # type: ignore[attr-defined]

    parser.add_argument(
        "--repository",
        required=True,
        help="The github repository (owner/name). Used for building the links "
        "to the repository.",
    )

    parser.add_argument(
        "--versioning-scheme",
        help="Versioning scheme to use for parsing and handling version "
        f"information. Choices are {', '.join(VERSIONING_SCHEMES.keys())}. "
        "Default: %(default)s",
        default="pep440",
        type=versioning_scheme_argument_type,
    )

    parser.add_argument(
        "--current-version",
        help="Version to start looking for changes. All commits since this "
        "releases are take into account for creating the changelog text.",
    )

    parser.add_argument(
        "--next-version",
        "--release-version",
        dest="next_version",
        help="The planned release version",
    )

    parser.add_argument(
        "--git-tag-prefix",
        default="v",
        help="Prefix for git tag versions. Used to determine existing "
        "releases. Default: %(default)s",
    )

    parser.add_argument(
        "--output",
        "-o",
        type=Path,
        help="Write changelog to this file.",
    ).complete = shtab.FILE  # type: ignore[attr-defined]

    parser.add_argument(
        "--quiet",
        "-q",
        action="store_true",
        help="Don't print messages to the terminal",
    )

    parsed_args = parser.parse_args(args=args)

    scheme: VersioningScheme = parsed_args.versioning_scheme
    current_version = getattr(parsed_args, "current_version", None)
    if current_version:
        parsed_args.current_version = scheme.parse_version(current_version)

    next_version = getattr(parsed_args, "next_version", None)
    if next_version:
        parsed_args.next_version = scheme.parse_version(next_version)

    return parsed_args



# Copyright (C) 2022 - 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from contextlib import AbstractAsyncContextManager
from types import TracebackType
from typing import Optional, Type

import httpx

from pontos.github.api.artifacts import GitHubAsyncRESTArtifacts
from pontos.github.api.billing import GitHubAsyncRESTBilling
from pontos.github.api.branch import GitHubAsyncRESTBranches
from pontos.github.api.client import GitHubAsyncRESTClient
from pontos.github.api.code_scanning import GitHubAsyncRESTCodeScanning
from pontos.github.api.contents import GitHubAsyncRESTContent
from pontos.github.api.dependabot import GitHubAsyncRESTDependabot
from pontos.github.api.helper import (
    DEFAULT_GITHUB_API_URL,
    DEFAULT_TIMEOUT_CONFIG,
)
from pontos.github.api.labels import GitHubAsyncRESTLabels
from pontos.github.api.organizations import GitHubAsyncRESTOrganizations
from pontos.github.api.packages import GitHubAsyncRESTPackages
from pontos.github.api.pull_requests import GitHubAsyncRESTPullRequests
from pontos.github.api.release import GitHubAsyncRESTReleases
from pontos.github.api.repositories import GitHubAsyncRESTRepositories
from pontos.github.api.search import GitHubAsyncRESTSearch
from pontos.github.api.secret_scanning import GitHubAsyncRESTSecretScanning
from pontos.github.api.tags import GitHubAsyncRESTTags
from pontos.github.api.teams import GitHubAsyncRESTTeams
from pontos.github.api.users import GitHubAsyncRESTUsers
from pontos.github.api.workflows import GitHubAsyncRESTWorkflows
from pontos.helper import deprecated


class GitHubAsyncRESTApi(AbstractAsyncContextManager):
    """
    A asynchronous GitHub REST API.

    Should be used as an async context manager.

    Example:
        .. code-block:: python

            from pontos.github.api import GitHubAsyncRESTApi

            async with GitHubAsyncRESTApi(token) as api:
                repositories = await api.organizations.get_repositories("foo")
    """

    def __init__(
        self,
        token: Optional[str] = None,
        url: Optional[str] = DEFAULT_GITHUB_API_URL,
        *,
        timeout: Optional[httpx.Timeout] = DEFAULT_TIMEOUT_CONFIG,
    ) -> None:
        """
        Args:
            token: GitHub API token
            url: GitHub URL
            timeout: Timeout settings to use
        """
        self._client = GitHubAsyncRESTClient(token, url, timeout=timeout)

    @property
    def organizations(self) -> GitHubAsyncRESTOrganizations:
        """
        Organizations related API
        """
        return GitHubAsyncRESTOrganizations(self._client)

    @property
    def artifacts(self) -> GitHubAsyncRESTArtifacts:
        """
        Artifacts related API
        """
        return GitHubAsyncRESTArtifacts(self._client)

    @property
    def billing(self) -> GitHubAsyncRESTBilling:
        """
        Billing related API
        """
        return GitHubAsyncRESTBilling(self._client)

    @property
    def branches(self) -> GitHubAsyncRESTBranches:
        """
        Branches related API
        """
        return GitHubAsyncRESTBranches(self._client)

    @property
    def code_scanning(self) -> GitHubAsyncRESTCodeScanning:
        """
        Code scanning related API
        """
        return GitHubAsyncRESTCodeScanning(self._client)

    @property
    def contents(self) -> GitHubAsyncRESTContent:
        """
        Contents related API
        """
        return GitHubAsyncRESTContent(self._client)

    @property
    def dependabot(self) -> GitHubAsyncRESTDependabot:
        """
        Dependabot related API
        """
        return GitHubAsyncRESTDependabot(self._client)

    @property
    def labels(self) -> GitHubAsyncRESTLabels:
        """
        Labels related API
        """
        return GitHubAsyncRESTLabels(self._client)

    @property
    def packages(self) -> GitHubAsyncRESTPackages:
        """
        Packages related API
        """
        return GitHubAsyncRESTPackages(self._client)

    @property
    @deprecated(
        since="23.3.4",
        reason="The pulls property is obsolete. Please use pull_requests "
        "instead.",
    )
    def pulls(self) -> GitHubAsyncRESTPullRequests:
        """
        Pull Requests related API

        .. deprecated:: 23.3.4

            Use :py:attr:`pull_requests` instead.
        """
        return GitHubAsyncRESTPullRequests(self._client)

    @property
    def pull_requests(self) -> GitHubAsyncRESTPullRequests:
        """
        Pull Requests related API
        """
        return GitHubAsyncRESTPullRequests(self._client)

    @property
    def releases(self) -> GitHubAsyncRESTReleases:
        """
        Releases related API
        """
        return GitHubAsyncRESTReleases(self._client)

    @property
    def workflows(self) -> GitHubAsyncRESTWorkflows:
        """
        Workflows related API
        """
        return GitHubAsyncRESTWorkflows(self._client)

    @property
    def repositories(self) -> GitHubAsyncRESTRepositories:
        """
        Repositories related API
        """
        return GitHubAsyncRESTRepositories(self._client)

    @property
    def secret_scanning(self) -> GitHubAsyncRESTSecretScanning:
        """
        Secret scanning related API
        """
        return GitHubAsyncRESTSecretScanning(self._client)

    @property
    def search(self) -> GitHubAsyncRESTSearch:
        """
        Search related API
        """
        return GitHubAsyncRESTSearch(self._client)

    @property
    def teams(self) -> GitHubAsyncRESTTeams:
        """
        Teams related API
        """
        return GitHubAsyncRESTTeams(self._client)

    @property
    def tags(self) -> GitHubAsyncRESTTags:
        """
        Tags related API
        """
        return GitHubAsyncRESTTags(self._client)

    @property
    def users(self) -> GitHubAsyncRESTUsers:
        """
        Users related API
        """
        return GitHubAsyncRESTUsers(self._client)

    async def __aenter__(self) -> "GitHubAsyncRESTApi":
        await self._client.__aenter__()
        return self

    async def __aexit__(
        self,
        exc_type: Optional[Type[BaseException]],
        exc_value: Optional[BaseException],
        traceback: Optional[TracebackType],
    ) -> Optional[bool]:
        return await self._client.__aexit__(exc_type, exc_value, traceback)



# Copyright (C) 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#


from typing import List, Literal, Type, Union

from ._errors import ProjectError
from ._version import Version, VersionUpdate
from .commands import VersionCommand, get_commands
from .schemes import VersioningScheme

__all__ = ("Project",)


class Project:
    """
    A project for handling versioning

    Example:
        .. code-block:: python

            from pontos.version.scheme import PEP440VersioningScheme
            from pontos.version.project import Project

            project = Project(PEP440VersioningScheme)
    """

    def __init__(
        self, versioning_scheme: Union[VersioningScheme, Type[VersioningScheme]]
    ) -> None:
        """
        Creates a new project instance

        Args:
            versioning_scheme: Scheme for version handling

        Raises:
            ProjectError: If no fitting VersionCommand could be found
        """
        self._versioning_scheme = versioning_scheme
        self._commands = self._gather_commands()

    def _gather_commands(self) -> List[VersionCommand]:
        """
        Initialize the project with the fitting VersionCommands of the current
        working directory

        Raises:
            ProjectError: If no fitting VersionCommand could be found
        """
        commands = []
        for cmd in get_commands():
            command = cmd(versioning_scheme=self._versioning_scheme)
            if command.project_found():
                commands.append(command)

        if not commands:
            raise ProjectError("No project settings file found")

        return commands

    def update_version(
        self, new_version: Version, *, force: bool = False
    ) -> VersionUpdate:
        """
        Update the current version of this project

        Args:
            new_version: Use this version in the update
            force: Force updating the version even if the current version is the
                same as the new version

        Returns:
            The version update including the changed files
        """
        update = self._commands[0].update_version(new_version, force=force)
        for cmd in self._commands[1:]:
            next_update = cmd.update_version(new_version, force=force)
            update.changed_files.extend(next_update.changed_files)

        return update

    def get_current_version(self) -> Version:
        """
        Get the current version of the project

        Returns:
            The current version
        """
        return self._commands[0].get_current_version()

    def verify_version(
        self, version: Union[Literal["current"], Version]
    ) -> None:
        """
        Verify the current version of this project

        Args:
            version: Version to check against the current applied version of
                this project. If version is "current" the command should verify
                if all version information is consistent, for example if the
                version information in several files is the same.
        """
        for cmd in self._commands:
            cmd.verify_version(version)



# Copyright (C) 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from enum import Enum
from pathlib import Path
from typing import Iterator


class Status(Enum):
    """
    Status of a file in git
    """

    UNMODIFIED = " "
    MODIFIED = "M"
    ADDED = "A"
    DELETED = "D"
    RENAMED = "R"
    COPIED = "C"
    UPDATED = "U"
    UNTRACKED = "?"
    IGNORED = "!"


class StatusEntry:
    """
    Status of a file in the git index and working tree.

    Implements the :py:class:`os.PathLike` protocol.

    Attributes:
        index: Status in the index
        working_tree: Status in the working tree
        path: Path to the file
        old_path: Set for renamed files
    """

    def __init__(self, status_string: str) -> None:
        status = status_string[:2]
        filename = status_string[3:]

        # Status in the index
        self.index = Status(status[0])
        # Status in the working directory
        self.working_tree = Status(status[1])

        if self.index == Status.RENAMED:
            new_filename, old_filename = filename.split("\0")
            self.path = Path(new_filename)
            self.old_path = Path(old_filename)
        else:
            # path of the file in git
            self.path = Path(filename)

    def __str__(self) -> str:
        return f"{self.index.value}{self.working_tree.value} {self.path}"

    def __repr__(self) -> str:
        return f"<StatusEntry {str(self)}>"

    def __fspath__(self):
        return self.path.__fspath__()


def parse_git_status(output: str) -> Iterator[StatusEntry]:
    output = output.rstrip("\0")
    if not output:
        return

    output_list = output.split("\0")
    while output_list:
        line = output_list.pop(0)
        if line[0] == Status.RENAMED.value:
            yield StatusEntry(f"{line}\0{output_list.pop(0)}")
        else:
            yield StatusEntry(line)



# SPDX-FileCopyrightText: 2022-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ._git import (
    DEFAULT_TAG_SORT_SUFFIX,
    ConfigScope,
    Git,
    GitError,
    MergeStrategy,
    ResetMode,
    TagSort,
)
from ._status import Status, StatusEntry

__all__ = (
    "DEFAULT_TAG_SORT_SUFFIX",
    "ConfigScope",
    "Git",
    "GitError",
    "MergeStrategy",
    "ResetMode",
    "Status",
    "StatusEntry",
    "TagSort",
)



# SPDX-FileCopyrightText: 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

from typing import AsyncIterator, Iterable, Optional, Union

from pontos.github.api.client import GitHubAsyncREST
from pontos.github.models.base import SortOrder
from pontos.github.models.secret_scanning import (
    AlertLocation,
    AlertSort,
    AlertState,
    CommitLocation,
    IssueBodyLocation,
    IssueCommentLocation,
    IssueTitleLocation,
    LocationType,
    Resolution,
    SecretScanningAlert,
)
from pontos.helper import enum_or_value


class GitHubAsyncRESTSecretScanning(GitHubAsyncREST):
    async def _alerts(
        self,
        api: str,
        *,
        state: Union[AlertState, str, None] = None,
        secret_types: Union[Iterable[str], None] = None,
        resolutions: Union[Iterable[str], None] = None,
        sort: Union[AlertSort, str] = AlertSort.CREATED,
        direction: Union[str, SortOrder] = SortOrder.DESC,
    ) -> AsyncIterator[SecretScanningAlert]:
        params = {"per_page": "100"}

        if state:
            params["state"] = enum_or_value(state)
        if secret_types:
            # as per REST api docu this param is passed as secret_type
            params["secret_type"] = ",".join(secret_types)
        if resolutions:
            # as per REST api docu this param is passed as resolution
            params["resolution"] = ",".join(resolutions)
        if sort:
            params["sort"] = enum_or_value(sort)
        if direction:
            params["direction"] = enum_or_value(direction)

        async for response in self._client.get_all(api, params=params):
            response.raise_for_status()

            for alert in response.json():
                yield SecretScanningAlert.from_dict(alert)

    async def enterprise_alerts(
        self,
        enterprise: str,
        *,
        state: Union[AlertState, str, None] = None,
        secret_types: Union[Iterable[str], None] = None,
        resolutions: Union[Iterable[str], None] = None,
        sort: Union[AlertSort, str] = AlertSort.CREATED,
        direction: Union[str, SortOrder] = SortOrder.DESC,
    ) -> AsyncIterator[SecretScanningAlert]:
        """
        Get the list of secret scanning alerts for all repositories of a GitHub
        enterprise

        https://docs.github.com/en/rest/secret-scanning/secret-scanning#list-secret-scanning-alerts-for-an-enterprise

        Args:
            enterprise: Name of the enterprise
            state: Filter alerts by state
            secret_types: List of secret types to return.
            resolutions: List secret scanning alerts with one of these provided
                resolutions
            sort: The property by which to sort the results. Default is to sort
                alerts by creation date.
            direction: The direction to sort the results by. Default is desc.

        Raises:
            HTTPStatusError: A httpx.HTTPStatusError is raised if the request
                failed.

        Returns:
            An async iterator yielding the secret scanning alerts

        Example:
            .. code-block:: python

                from pontos.github.api import GitHubAsyncRESTApi

                async with GitHubAsyncRESTApi(token) as api:
                    async for alert in api.secret_scanning.enterprise_alerts(
                        "my-enterprise"
                    ):
                        print(alert)
        """

        api = f"/enterprises/{enterprise}/secret-scanning/alerts"
        async for alert in self._alerts(
            api,
            state=state,
            secret_types=secret_types,
            resolutions=resolutions,
            sort=sort,
            direction=direction,
        ):
            yield alert

    async def organization_alerts(
        self,
        organization: str,
        *,
        state: Union[AlertState, str, None] = None,
        secret_types: Union[Iterable[str], None] = None,
        resolutions: Union[Iterable[str], None] = None,
        sort: Union[AlertSort, str] = AlertSort.CREATED,
        direction: Union[str, SortOrder] = SortOrder.DESC,
    ) -> AsyncIterator[SecretScanningAlert]:
        """
        Get the list of secret scanning alerts for all repositories of a GitHub
        organization

        https://docs.github.com/en/rest/secret-scanning/secret-scanning#list-secret-scanning-alerts-for-an-organization

        Args:
            organization: Name of the organization
            state: Filter alerts by state
            secret_types: List of secret types to return.
            resolutions: List secret scanning alerts with one of these provided
                resolutions
            sort: The property by which to sort the results. Default is to sort
                alerts by creation date.
            direction: The direction to sort the results by. Default is desc.

        Raises:
            HTTPStatusError: A httpx.HTTPStatusError is raised if the request
                failed.

        Returns:
            An async iterator yielding the secret scanning alerts

        Example:
            .. code-block:: python

                from pontos.github.api import GitHubAsyncRESTApi

                async with GitHubAsyncRESTApi(token) as api:
                    async for alert in api.secret_scanning.organization_alerts(
                        "my-org"
                    ):
                        print(alert)
        """

        api = f"/orgs/{organization}/secret-scanning/alerts"
        async for alert in self._alerts(
            api,
            state=state,
            secret_types=secret_types,
            resolutions=resolutions,
            sort=sort,
            direction=direction,
        ):
            yield alert

    async def alerts(
        self,
        repo: str,
        *,
        state: Union[AlertState, str, None] = None,
        secret_types: Union[Iterable[str], None] = None,
        resolutions: Union[Iterable[str], None] = None,
        sort: Union[AlertSort, str] = AlertSort.CREATED,
        direction: Union[str, SortOrder] = SortOrder.DESC,
    ) -> AsyncIterator[SecretScanningAlert]:
        """
        Get the list of secret scanning alerts for a repository

        https://docs.github.com/en/rest/secret-scanning/secret-scanning#list-secret-scanning-alerts-for-a-repository

        Args:
            repo: GitHub repository (owner/name)
            state: Filter alerts by state
            secret_types: List of secret types to return.
            resolutions: List secret scanning alerts with one of these provided
                resolutions
            sort: The property by which to sort the results. Default is to sort
                alerts by creation date.
            direction: The direction to sort the results by. Default is desc.

        Raises:
            HTTPStatusError: A httpx.HTTPStatusError is raised if the request
                failed.

        Returns:
            An async iterator yielding the secret scanning alerts

        Example:
            .. code-block:: python

                from pontos.github.api import GitHubAsyncRESTApi

                async with GitHubAsyncRESTApi(token) as api:
                    async for alert in api.secret_scanning.alerts(
                        "my-org/my-repo"
                    ):
                        print(alert)
        """

        api = f"/repos/{repo}/secret-scanning/alerts"
        async for alert in self._alerts(
            api,
            state=state,
            secret_types=secret_types,
            resolutions=resolutions,
            sort=sort,
            direction=direction,
        ):
            yield alert

    async def alert(
        self,
        repo: str,
        alert_number: Union[str, int],
    ) -> SecretScanningAlert:
        """
        Get a single secret scanning alert

        https://docs.github.com/en/rest/secret-scanning/secret-scanning#get-a-secret-scanning-alert

        Args:
            repo: GitHub repository (owner/name)
            alert_number: The number that identifies a secret scanning alert in
                its repository

        Raises:
            HTTPStatusError: A httpx.HTTPStatusError is raised if the request
                failed.

        Returns:
            Secret scanning alert information

        Example:
            .. code-block:: python

                from pontos.github.api import GitHubAsyncRESTApi

                async with GitHubAsyncRESTApi(token) as api:
                    alert = await api.secret_scanning.alert("foo/bar", 123)
        """
        api = f"/repos/{repo}/secret-scanning/alerts/{alert_number}"
        response = await self._client.get(api)
        response.raise_for_status()
        return SecretScanningAlert.from_dict(response.json())

    async def update_alert(
        self,
        repo: str,
        alert_number: Union[str, int],
        state: Union[AlertState, str],
        *,
        resolution: Union[Resolution, str, None] = None,
        resolution_comment: Optional[str] = None,
    ) -> SecretScanningAlert:
        """
        Update a single secret scanning alert

        https://docs.github.com/en/rest/secret-scanning/secret-scanning#update-a-secret-scanning-alert

        Args:
            repo: GitHub repository (owner/name)
            alert_number: The number that identifies a secret scanning alert in
                its repository
            state: The state of the alert
            resolution: Required when the state is resolved. The reason for
                resolving the alert
            resolution_comment: An optional comment when closing an alert.
                Cannot be updated or deleted.

        Raises:
            HTTPStatusError: A httpx.HTTPStatusError is raised if the request
                failed.

        Returns:
            Secret scanning alert information

        Example:
            .. code-block:: python

                from pontos.github.api import GitHubAsyncRESTApi
                from pontos.github.models.secret_scanning import (
                    AlertState,
                    Resolution,
                )

                async with GitHubAsyncRESTApi(token) as api:
                    alert = await api.secret_scanning.update_alert(
                        "foo/bar",
                        123,
                        AlertState.RESOLVED,
                        resolution=Resolution.WONT_FIX,
                        resolution_comment="Not applicable",
                    )
        """
        api = f"/repos/{repo}/secret-scanning/alerts/{alert_number}"

        data = {"state": enum_or_value(state)}
        if resolution:
            data["resolution"] = enum_or_value(resolution)
        if resolution_comment:
            data["resolution_comment"] = resolution_comment

        response = await self._client.patch(api, data=data)
        response.raise_for_status()
        return SecretScanningAlert.from_dict(response.json())

    async def locations(
        self,
        repo: str,
        alert_number: Union[str, int],
    ) -> AsyncIterator[AlertLocation]:
        """
        Lists all locations for a given secret scanning alert for an eligible
        repository

        https://docs.github.com/en/rest/secret-scanning/secret-scanning#list-locations-for-a-secret-scanning-alert

        Args:
            repo: GitHub repository (owner/name)
            alert_number: The number that identifies a secret scanning alert in
                its repository

        Raises:
            HTTPStatusError: A httpx.HTTPStatusError is raised if the request
                failed.

        Returns:
            An async iterator yielding the secret scanning alert locations

        Example:
            .. code-block:: python

                from pontos.github.api import GitHubAsyncRESTApi

                async with GitHubAsyncRESTApi(token) as api:
                    async for location in await api.secret_scanning.locations(
                        "foo/bar",
                        123,
                    ):
                        print(location)
        """
        api = f"/repos/{repo}/secret-scanning/alerts/{alert_number}/locations"
        params = {"per_page": "100"}

        async for response in self._client.get_all(api, params=params):
            response.raise_for_status()

            for location in response.json():
                location_type = location["type"]
                location_details = location["details"]
                if location_type == LocationType.COMMIT.value:
                    yield AlertLocation(
                        type=LocationType.COMMIT,
                        details=CommitLocation.from_dict(location_details),
                    )
                elif location_type == LocationType.ISSUE_BODY.value:
                    yield AlertLocation(
                        type=LocationType.ISSUE_BODY,
                        details=IssueBodyLocation.from_dict(location_details),
                    )
                elif location_type == LocationType.ISSUE_COMMENT.value:
                    yield AlertLocation(
                        type=LocationType.ISSUE_COMMENT,
                        details=IssueCommentLocation.from_dict(
                            location_details
                        ),
                    )
                elif location_type == LocationType.ISSUE_TITLE.value:
                    yield AlertLocation(
                        type=LocationType.ISSUE_TITLE,
                        details=IssueTitleLocation.from_dict(location_details),
                    )



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from argparse import ArgumentParser, Namespace

from pontos.github.api import GitHubAsyncRESTApi


def add_script_arguments(parser: ArgumentParser) -> None:
    parser.add_argument("repo")
    parser.add_argument("branch")


async def github_script(api: GitHubAsyncRESTApi, args: Namespace) -> int:
    # draft script for checking the branch protection
    branch_protection = await api.branches.protection_rules(
        args.repo, args.branch
    )
    if branch_protection:
        print(
            f"Branch protection enabled for the '{args.branch}' branch of the '{args.repo}' repository."
        )
        return 0
    else:
        print(
            f"Branch protection NOT enabled for the '{args.branch}' branch of the '{args.repo}' repository."
        )
        return 1



# SPDX-FileCopyrightText: 2022-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import io
import unittest
from argparse import ArgumentParser
from contextlib import redirect_stderr

from pontos.github.script.errors import GitHubScriptError
from pontos.github.script.load import (
    load_script,
    run_add_arguments_function,
    run_github_script_function,
)
from pontos.testing import temp_file, temp_python_module
from tests import IsolatedAsyncioTestCase


class LoadScriptTestCase(unittest.TestCase):
    def test_load_script(self):
        with (
            temp_file("def foo():\n\treturn 1", name="foo.py") as f,
            load_script(f) as module,
        ):
            self.assertIsNotNone(module)
            self.assertIsNotNone(module.foo)
            self.assertEqual(module.foo(), 1)

    def test_load_script_module(self):
        with (
            temp_python_module(
                "def foo():\n\treturn 1", name="github-foo-script"
            ),
            load_script("github-foo-script") as module,
        ):
            self.assertIsNotNone(module)
            self.assertIsNotNone(module.foo)
            self.assertEqual(module.foo(), 1)

    def test_load_script_failure(self):
        with self.assertRaisesRegex(
            ModuleNotFoundError, "No module named 'baz'"
        ):
            with load_script("foo/bar/baz.py"):
                pass


class RunAddArgumentsFunction(unittest.TestCase):
    def test_add_arguments_function(self):
        with (
            temp_file(
                "def add_script_arguments(parser):\n\t"
                "parser.add_argument('--foo')",
                name="foo.py",
            ) as f,
            load_script(f) as module,
        ):
            parser = ArgumentParser()
            run_add_arguments_function(module, parser)
            args = parser.parse_args(["--foo", "123"])
            self.assertEqual(args.foo, "123")

    def test_no_add_arguments_function(self):
        with (
            temp_file(
                "def foo():\n\tpass",
                name="foo.py",
            ) as f,
            load_script(f) as module,
        ):
            parser = ArgumentParser()
            run_add_arguments_function(module, parser)
            with self.assertRaises(SystemExit), redirect_stderr(io.StringIO()):
                parser.parse_args(["--foo", "123"])


class RunGithubScriptFunctionTestCase(IsolatedAsyncioTestCase):
    def test_run_async_github_script_function(self):
        with (
            temp_file(
                "async def github_script(api, args):\n\treturn 1",
                name="foo.py",
            ) as f,
            load_script(f) as module,
        ):
            self.assertEqual(
                run_github_script_function(module, "123", 123, {}), 1
            )

    def test_sync_github_script_function_not_supported(self):
        with (
            temp_file(
                "def github_script(api, args):\n\treturn 1",
                name="foo.py",
            ) as f,
            load_script(f) as module,
        ):
            with self.assertRaises(GitHubScriptError):
                run_github_script_function(module, "123", 123, {})

    def test_no_github_script_function(self):
        with (
            temp_file(
                "def foo():\n\tpass",
                name="foo.py",
            ) as f,
            load_script(f) as module,
        ):
            with self.assertRaises(GitHubScriptError):
                run_github_script_function(module, "123", 123, {})



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

from argparse import ArgumentParser, Namespace
from typing import Optional, Sequence

import shtab


def cpe_matches_parse(args: Optional[Sequence[str]] = None) -> Namespace:
    parser = ArgumentParser()
    shtab.add_argument_to(parser)
    parser.add_argument("--token", help="API key to use for querying.")
    parser.add_argument("--cve-id", help="Get matches for a specific CVE")
    parser.add_argument(
        "--number", "-n", metavar="N", help="Request only N matches", type=int
    )
    parser.add_argument(
        "--start",
        "-s",
        help="Index of the first match to request.",
        type=int,
    )
    return parser.parse_args(args)


def cpe_match_parse(args: Optional[Sequence[str]] = None) -> Namespace:
    parser = ArgumentParser()
    shtab.add_argument_to(parser)
    parser.add_argument("--token", help="API key to use for querying.")
    parser.add_argument(
        "--match-criteria-id",
        help="Get the match string with the given matchCriteriaId ",
    )
    return parser.parse_args(args)



# SPDX-FileCopyrightText: 2022-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

# pylint: disable=protected-access

import unittest
from datetime import datetime
from typing import Any, Iterator
from unittest.mock import AsyncMock, MagicMock, call, patch

from httpx import AsyncClient, Response

from pontos.nvd.api import (
    JSON,
    InvalidState,
    NoMoreResults,
    NVDApi,
    NVDResults,
    convert_camel_case,
    format_date,
)
from tests import IsolatedAsyncioTestCase, aiter, anext


class ConvertCamelCaseTestCase(unittest.TestCase):
    def test_convert(self):
        data = {
            "someValue": 123,
            "otherValue": "bar",
        }

        converted = convert_camel_case(data)
        self.assertEqual(converted["some_value"], 123)
        self.assertEqual(converted["other_value"], "bar")


class FormatDateTestCase(unittest.TestCase):
    def test_format_date(self):
        dt = datetime(2022, 12, 10, 10, 0, 12, 123)
        fd = format_date(dt)

        self.assertEqual(fd, "2022-12-10T10:00:12")


class NVDApiTestCase(IsolatedAsyncioTestCase):
    @patch("pontos.nvd.api.AsyncClient", spec=AsyncClient)
    async def test_context_manager(self, async_client: MagicMock):
        http_client = AsyncMock()
        async_client.return_value = http_client
        api = NVDApi("https://foo.bar/baz", token="token")

        async with api:
            pass

        http_client.__aenter__.assert_awaited_once()
        http_client.__aexit__.assert_awaited_once()

    @patch("pontos.nvd.api.AsyncClient", spec=AsyncClient)
    async def test_get_without_token(self, async_client: MagicMock):
        http_client = AsyncMock()
        async_client.return_value = http_client
        api = NVDApi("https://foo.bar/baz")

        await api._get()

        http_client.get.assert_awaited_once_with(
            "https://foo.bar/baz", headers={}, params=None
        )

    @patch("pontos.nvd.api.AsyncClient", spec=AsyncClient)
    async def test_get_with_token(self, async_client: MagicMock):
        http_client = AsyncMock()
        async_client.return_value = http_client
        api = NVDApi("https://foo.bar/baz", token="token")

        await api._get()

        http_client.get.assert_awaited_once_with(
            "https://foo.bar/baz", headers={"apiKey": "token"}, params=None
        )

    @patch("pontos.nvd.api.time.monotonic", autospec=True)
    @patch("pontos.nvd.api.asyncio.sleep", autospec=True)
    @patch("pontos.nvd.api.AsyncClient", spec=AsyncClient)
    async def test_rate_limit(
        self,
        async_client: MagicMock,
        sleep_mock: MagicMock,
        monotonic_mock: MagicMock,
    ):
        http_client = AsyncMock()
        async_client.return_value = http_client
        monotonic_mock.side_effect = [0.0, 10.0, 11.0]

        api = NVDApi("https://foo.bar/baz")

        await api._get()
        await api._get()
        await api._get()
        await api._get()
        await api._get()

        sleep_mock.assert_not_called()

        await api._get()

        sleep_mock.assert_called_once_with(20.0)

    @patch("pontos.nvd.api.asyncio.sleep", autospec=True)
    @patch("pontos.nvd.api.AsyncClient", spec=AsyncClient)
    async def test_no_rate_limit(
        self, async_client: MagicMock, sleep_mock: MagicMock
    ):
        http_client = AsyncMock()
        async_client.return_value = http_client
        api = NVDApi("https://foo.bar/baz", rate_limit=False)

        await api._get()
        await api._get()
        await api._get()
        await api._get()
        await api._get()

        sleep_mock.assert_not_called()

        await api._get()

        sleep_mock.assert_not_called()

    @patch("pontos.nvd.api.asyncio.sleep", autospec=True)
    @patch("pontos.nvd.api.AsyncClient", spec=AsyncClient)
    async def test_retry(
        self,
        async_client: MagicMock,
        sleep_mock: MagicMock,
    ):
        response_mocks = [
            MagicMock(spec=Response, is_server_error=True),
            MagicMock(spec=Response, is_server_error=True),
            MagicMock(spec=Response, is_server_error=True),
            MagicMock(spec=Response, is_server_error=False),
        ]
        http_client = AsyncMock()
        http_client.get.side_effect = response_mocks
        async_client.return_value = http_client

        api = NVDApi("https://foo.bar/baz", request_attempts=4)

        result = await api._get()

        calls = [call(2.0), call(4.0), call(8.0)]
        sleep_mock.assert_has_calls(calls)
        self.assertFalse(result.is_server_error)

    @patch("pontos.nvd.api.asyncio.sleep", autospec=True)
    @patch("pontos.nvd.api.AsyncClient", spec=AsyncClient)
    async def test_no_retry(
        self,
        async_client: MagicMock,
        sleep_mock: MagicMock,
    ):
        response_mock = MagicMock(spec=Response)
        response_mock.is_server_error = False

        http_client = AsyncMock()
        http_client.get.return_value = response_mock
        async_client.return_value = http_client

        api = NVDApi("https://foo.bar/baz")

        result = await api._get()

        sleep_mock.assert_not_called()
        self.assertFalse(result.is_server_error)


class Result:
    def __init__(self, value: int) -> None:
        self.value = value


def result_func(data: JSON) -> Iterator[Result]:
    return (Result(d) for d in data["values"])  # type: ignore


class NVDResultsTestCase(IsolatedAsyncioTestCase):
    async def test_items(self):
        response_mock = MagicMock(spec=Response)
        response_mock.json.side_effect = [
            {
                "values": [1, 2, 3],
                "total_results": 6,
                "results_per_page": 3,
            },
            {
                "values": [4, 5, 6],
                "total_results": 6,
                "results_per_page": 3,
            },
        ]
        api_mock = AsyncMock(spec=NVDApi)
        api_mock._get.return_value = response_mock

        results: NVDResults[Result] = NVDResults(
            api_mock,
            {},
            result_func,
        )

        it = aiter(results.items())

        result = await anext(it)
        self.assertEqual(result.value, 1)

        result = await anext(it)
        self.assertEqual(result.value, 2)

        result = await anext(it)
        self.assertEqual(result.value, 3)

        result = await anext(it)
        self.assertEqual(result.value, 4)

        result = await anext(it)
        self.assertEqual(result.value, 5)

        result = await anext(it)
        self.assertEqual(result.value, 6)

        with self.assertRaises(StopAsyncIteration):
            await anext(it)

    async def test_items_no_results(self):
        response_mock = MagicMock(spec=Response)
        response_mock.json.side_effect = [
            {
                "values": [],
                "total_results": 0,
                "results_per_page": 0,
            },
        ]
        api_mock = AsyncMock(spec=NVDApi)
        api_mock._get.return_value = response_mock

        results: NVDResults[Result] = NVDResults(
            api_mock,
            {},
            result_func,
        )

        it = aiter(results.items())

        with self.assertRaises(StopAsyncIteration):
            await anext(it)

    async def test_aiter(self):
        response_mock = MagicMock(spec=Response)
        response_mock.json.side_effect = [
            {
                "values": [1, 2, 3],
                "total_results": 6,
                "results_per_page": 3,
            },
            {
                "values": [4, 5, 6],
                "total_results": 6,
                "results_per_page": 3,
            },
        ]
        api_mock = AsyncMock(spec=NVDApi)
        api_mock._get.return_value = response_mock

        results: NVDResults[Result] = NVDResults(
            api_mock,
            {},
            result_func,
        )

        it = aiter(results)

        result = await anext(it)
        self.assertEqual(result.value, 1)

        result = await anext(it)
        self.assertEqual(result.value, 2)

        result = await anext(it)
        self.assertEqual(result.value, 3)

        result = await anext(it)
        self.assertEqual(result.value, 4)

        result = await anext(it)
        self.assertEqual(result.value, 5)

        result = await anext(it)
        self.assertEqual(result.value, 6)

        with self.assertRaises(StopAsyncIteration):
            await anext(it)

    async def test_len(self):
        response_mock = MagicMock(spec=Response)
        response_mock.json.return_value = {
            "values": [1, 2, 3],
            "total_results": 3,
            "results_per_page": 3,
        }
        api_mock = AsyncMock(spec=NVDApi)
        api_mock._get.return_value = response_mock

        results: NVDResults[Result] = NVDResults(
            api_mock,
            {},
            result_func,
        )

        with self.assertRaisesRegex(
            InvalidState, "NVDResults has not been awaited yet"
        ):
            len(results)

        await results

        self.assertEqual(len(results), 3)

    async def test_chunks(self):
        response_mock = MagicMock(spec=Response)
        response_mock.json.side_effect = [
            {
                "values": [1, 2, 3],
                "total_results": 6,
                "results_per_page": 3,
            },
            {
                "values": [4, 5, 6],
                "total_results": 6,
                "results_per_page": 3,
            },
        ]
        api_mock = AsyncMock(spec=NVDApi)
        api_mock._get.return_value = response_mock

        nvd_results: NVDResults[Result] = NVDResults(
            api_mock,
            {},
            result_func,
        )

        it = aiter(nvd_results.chunks())

        results = await anext(it)
        self.assertEqual([result.value for result in results], [1, 2, 3])

        results = await anext(it)
        self.assertEqual([result.value for result in results], [4, 5, 6])

        with self.assertRaises(StopAsyncIteration):
            await anext(it)

    async def test_json(self):
        response_mock = MagicMock(spec=Response)
        response_mock.json.side_effect = [
            {
                "values": [1, 2, 3],
                "total_results": 6,
                "results_per_page": 3,
            },
            {
                "values": [4, 5, 6],
                "total_results": 6,
                "results_per_page": 3,
            },
        ]
        api_mock = AsyncMock(spec=NVDApi)
        api_mock._get.return_value = response_mock

        nvd_results: NVDResults[Result] = NVDResults(
            api_mock,
            {},
            result_func,
        )

        json: dict[str, Any] = await nvd_results.json()  # type: ignore
        self.assertEqual(json["values"], [1, 2, 3])
        self.assertEqual(json["total_results"], 6)
        self.assertEqual(json["results_per_page"], 3)

        json: dict[str, Any] = await nvd_results.json()  # type: ignore
        self.assertEqual(json["values"], [4, 5, 6])
        self.assertEqual(json["total_results"], 6)
        self.assertEqual(json["results_per_page"], 3)

        self.assertIsNone(await nvd_results.json())

    async def test_await(self):
        response_mock = MagicMock(spec=Response)
        response_mock.json.side_effect = [
            {
                "values": [1, 2, 3],
                "total_results": 6,
                "results_per_page": 3,
            },
            {
                "values": [4, 5, 6],
                "total_results": 6,
                "results_per_page": 3,
            },
        ]
        api_mock = AsyncMock(spec=NVDApi)
        api_mock._get.return_value = response_mock

        nvd_results: NVDResults[Result] = NVDResults(
            api_mock,
            {},
            result_func,
        )

        await nvd_results
        self.assertEqual(len(nvd_results), 6)

        json: dict[str, Any] = await nvd_results.json()  # type: ignore
        self.assertEqual(json["values"], [1, 2, 3])
        self.assertEqual(json["total_results"], 6)
        self.assertEqual(json["results_per_page"], 3)

        await nvd_results
        json: dict[str, Any] = await nvd_results.json()  # type: ignore
        self.assertEqual(json["values"], [4, 5, 6])
        self.assertEqual(json["total_results"], 6)
        self.assertEqual(json["results_per_page"], 3)

        with self.assertRaises(NoMoreResults):
            await nvd_results

    async def test_mix_and_match(self):
        response_mock = MagicMock(spec=Response)
        response_mock.json.side_effect = [
            {
                "values": [1, 2, 3],
                "total_results": 6,
                "results_per_page": 3,
            },
            {
                "values": [4, 5, 6],
                "total_results": 6,
                "results_per_page": 3,
            },
        ]
        api_mock = AsyncMock(spec=NVDApi)
        api_mock._get.return_value = response_mock

        nvd_results: NVDResults[Result] = NVDResults(
            api_mock,
            {},
            result_func,
        )

        await nvd_results
        self.assertEqual(len(nvd_results), 6)

        json: dict[str, Any] = await nvd_results.json()  # type: ignore
        self.assertEqual(json["values"], [1, 2, 3])
        self.assertEqual(json["total_results"], 6)
        self.assertEqual(json["results_per_page"], 3)

        self.assertEqual(
            [result.value async for result in nvd_results], [1, 2, 3, 4, 5, 6]
        )

        json: dict[str, Any] = await nvd_results.json()  # type: ignore
        self.assertEqual(json["values"], [4, 5, 6])
        self.assertEqual(json["total_results"], 6)
        self.assertEqual(json["results_per_page"], 3)

    async def test_response_error(self):
        response_mock = MagicMock(spec=Response)
        response_mock.json.side_effect = [
            {
                "values": [1, 2, 3],
                "total_results": 6,
                "results_per_page": 3,
            },
        ]
        api_mock = AsyncMock(spec=NVDApi)
        api_mock._get.return_value = response_mock

        nvd_results: NVDResults[Result] = NVDResults(
            api_mock,
            {},
            result_func,
        )

        json = await nvd_results.json()
        self.assertEqual(json["values"], [1, 2, 3])  # type: ignore

        api_mock._get.assert_called_once_with(params={"startIndex": 0})

        response_mock.raise_for_status.side_effect = Exception("Server Error")

        api_mock.reset_mock()

        with self.assertRaises(Exception):
            json = await nvd_results.json()

        api_mock._get.assert_called_once_with(
            params={
                "startIndex": 3,
                "resultsPerPage": 3,
            }
        )

        response_mock.reset_mock(return_value=True, side_effect=True)
        api_mock.reset_mock()

        response_mock.json.side_effect = [
            {
                "values": [4, 5, 6],
                "total_results": 6,
                "results_per_page": 3,
            },
        ]

        json = await nvd_results.json()
        self.assertEqual(json["values"], [4, 5, 6])  # type: ignore

        api_mock._get.assert_called_once_with(
            params={
                "startIndex": 3,
                "resultsPerPage": 3,
            }
        )

    async def test_request_results_limit(self):
        response_mock = MagicMock(spec=Response)
        response_mock.json.side_effect = [
            {
                "values": [1, 2, 3, 4],
                "total_results": 5,
                "results_per_page": 4,
            },
            {
                "values": [5],
                "total_results": 5,
                "results_per_page": 1,
            },
        ]
        api_mock = AsyncMock(spec=NVDApi)
        api_mock._get.return_value = response_mock

        nvd_results: NVDResults[Result] = NVDResults(
            api_mock,
            {},
            result_func,
            request_results=5,
        )

        json: dict[str, Any] = await nvd_results.json()  # type: ignore
        self.assertEqual(json["values"], [1, 2, 3, 4])
        self.assertEqual(json["total_results"], 5)
        self.assertEqual(json["results_per_page"], 4)

        api_mock._get.assert_called_once_with(params={"startIndex": 0})
        api_mock.reset_mock()

        json: dict[str, Any] = await nvd_results.json()  # type: ignore
        self.assertEqual(json["values"], [5])
        self.assertEqual(json["total_results"], 5)
        self.assertEqual(json["results_per_page"], 1)

        api_mock._get.assert_called_once_with(
            params={"startIndex": 4, "resultsPerPage": 1}
        )

    async def test_repr(self):
        response_mock = MagicMock(spec=Response)
        response_mock.json.side_effect = [
            {
                "values": [1, 2, 3, 4],
                "total_results": 5,
                "results_per_page": 4,
            },
            {
                "values": [5],
                "total_results": 5,
                "results_per_page": 1,
            },
        ]
        response_mock.url = "https://some.url&startIndex=0"
        api_mock = AsyncMock(spec=NVDApi)
        api_mock._get.return_value = response_mock

        nvd_results: NVDResults[Result] = NVDResults(
            api_mock,
            {},
            result_func,
        )

        await nvd_results

        self.assertEqual(
            repr(nvd_results),
            '<NVDResults url="https://some.url&startIndex=0" '
            "total_results=5 start_index=0 current_index=4 "
            "results_per_page=None>",
        )



# SPDX-FileCopyrightText: 2022-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from dataclasses import dataclass, field
from datetime import datetime
from typing import List, Optional

from pontos.github.models.base import GitHubModel, User
from pontos.models import StrEnum

__all__ = (
    "Release",
    "ReleaseAsset",
    "ReleaseAssetState",
    "ReleaseReactions",
)


class ReleaseAssetState(StrEnum):
    """
    State of a release asset

    Attributes:
        UPLOADED: Uploaded
        OPEN: Open
    """

    UPLOADED = "uploaded"
    OPEN = "open"


@dataclass
class ReleaseAsset(GitHubModel):
    """
    A GitHub release asset model

    Attributes:
        url: URL of the release asset
        browser_download_url: Direct URL to download the asset from
        id: ID of the asset
        node_id: Node ID of the asset
        name: Name of the asset
        state: State of the asset
        content_type: MIME content type of the asset
        size: Size of the asset
        download_count: Number of downloads
        created_at: Creation date
        updated_at: Upload date
        label: Label of the asset
        uploader: User who uploaded the asset
    """

    url: str
    browser_download_url: str
    id: int
    node_id: str
    name: str
    state: ReleaseAssetState
    content_type: str
    size: int
    download_count: int
    created_at: datetime
    updated_at: datetime
    label: Optional[str] = None
    uploader: Optional[User] = None


@dataclass
class ReleaseReactions(GitHubModel):
    """
    Reactions to a GitHub release

    Attributes:
        url: URL to the release reactions
        total_count: Total number of reactions
        laugh: Number of user reacted with laugh
        confused: Number of user reacted with confused
        heart: Number of user reacted with heart
        hooray: Number of user reacted with hooray
        eyes: Number of user reacted with eyes
        rocket: Number of user reacted with rocket
    """

    url: str
    total_count: int
    laugh: int
    confused: int
    heart: int
    hooray: int
    eyes: int
    rocket: int


@dataclass
class Release(GitHubModel):
    """
    A GitHub release model

    Attributes:
        assets_url: URL to the release assets
        created_at: Creation Date
        draft: True if the release is a draft
        html_url: URL to the web page of the release
        id: ID of the release
        node_id: Node ID of the release
        prerelease: True if the release is a pre release
        tag_name: Name of the tag referenced by the release
        target_commitish: Git commit ID of the tag references by the release
        upload_url: URL to upload release assets to
        url: URL of the release
        assets: Information about the release assets
        author: User created the release
        body_html: Body of the release as HTML
        body_text: Body of the release as text
        body: Body of the release
        discussion_url: URL to the release discussion
        mentions_count:
        name: Name of the release
        published_at: Publication date of the release
        reactions: Reaction information
        tarball_url: URL to the tarball archive of the release
        zipball_url: URL to the zip archive of the release
    """

    assets_url: str
    created_at: datetime
    draft: bool
    html_url: str
    id: int
    node_id: str
    prerelease: bool
    tag_name: str
    target_commitish: str
    upload_url: str
    url: str
    assets: List[ReleaseAsset] = field(default_factory=list)
    author: Optional[User] = None
    body_html: Optional[str] = None
    body_text: Optional[str] = None
    body: Optional[str] = None
    discussion_url: Optional[str] = None
    mentions_count: Optional[int] = None
    name: Optional[str] = None
    published_at: Optional[datetime] = None
    reactions: Optional[ReleaseReactions] = None
    tarball_url: Optional[str] = None
    zipball_url: Optional[str] = None



# SPDX-FileCopyrightText: 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

# pylint: disable=line-too-long

from typing import Any, Dict, Optional


def get_cve_data(data: Optional[Dict[str, Any]] = None) -> Dict[str, Any]:
    cve = {
        "id": "CVE-2022-45536",
        "source_identifier": "cve@mitre.org",
        "published": "2022-11-22T21:15:11.103",
        "last_modified": "2022-11-23T16:02:07.367",
        "vuln_status": "Analyzed",
        "descriptions": [
            {
                "lang": "en",
                "value": "AeroCMS v0.0.1 was discovered to contain a SQL "
                "Injection vulnerability via the id parameter at "
                "\\admin\\post_comments.php. This vulnerability allows "
                "attackers to access database information.",
            }
        ],
        "references": [
            {
                "url": "https://github.com/rdyx0/CVE/blob/master/AeroCMS/AeroCMS-v0.0.1-SQLi/post_comments_sql_injection/post_comments_sql_injection.md",
                "source": "cve@mitre.org",
                "tags": ["Exploit", "Third Party Advisory"],
            },
            {
                "url": "https://rdyx0.github.io/2018/09/07/AeroCMS-v0.0.1-SQLi%20post_comments_sql_injection/",
                "source": "cve@mitre.org",
                "tags": ["Exploit", "Third Party Advisory"],
            },
        ],
    }
    if data:
        cve.update(data)
    return cve


def get_cpe_data(update: Optional[Dict[str, Any]] = None) -> Dict[str, Any]:
    data = {
        "deprecated": False,
        "cpe_name": "cpe:2.3:o:microsoft:windows_10_22h2:-:*:*:*:*:*:arm64:*",
        "cpe_name_id": "9BAECDB2-614D-4E9C-9936-190C30246F03",
        "last_modified": "2022-12-09T18:15:16.973",
        "created": "2022-12-09T16:20:06.943",
    }

    if update:
        data.update(update)
    return data


def get_cpe_match_data(
    update: Optional[Dict[str, Any]] = None
) -> Dict[str, Any]:
    data = {
        "cpe_last_modified": "2019-07-22T16:37:38.133",
        "created": "2019-06-17T09:16:33.960",
        "criteria": "cpe:2.3:a:sun:jre:*:update3:*:*:*:*:*:*",
        "last_modified": "2019-06-17T09:16:44.000",
        "match_criteria_id": "EAB2C9C2-F685-450B-9980-553966FC3B63",
        "matches": [
            {
                "cpe_name": "cpe:2.3:a:sun:jre:1.3.0:update3:*:*:*:*:*:*",
                "cpe_name_id": "2D284534-DA21-43D5-9D89-07F19AE400EA",
            },
            {
                "cpe_name": "cpe:2.3:a:sun:jre:1.4.1:update3:*:*:*:*:*:*",
                "cpe_name_id": "CE55E1DF-8EA2-41EA-9C51-1BAE728CA094",
            },
            {
                "cpe_name": "cpe:2.3:a:sun:jre:1.4.2:update3:*:*:*:*:*:*",
                "cpe_name_id": "A09C4E47-6548-40C5-8458-5C07C3292C86",
            },
            {
                "cpe_name": "cpe:2.3:a:sun:jre:1.5.0:update3:*:*:*:*:*:*",
                "cpe_name_id": "C484A93A-2677-4501-A6E0-E4ADFFFB549E",
            },
            {
                "cpe_name": "cpe:2.3:a:sun:jre:1.6.0:update3:*:*:*:*:*:*",
                "cpe_name_id": "C518A954-369E-453E-8E17-2AF639150115",
            },
        ],
        "status": "Active",
        "version_end_including": "1.6.0",
    }

    if update:
        data.update(update)
    return data


def get_cve_change_data(
    data: Optional[Dict[str, Any]] = None
) -> Dict[str, Any]:
    cve_changes = {
        "cve_id": "CVE-2022-0001",
        "event_name": "Initial Analysis",
        "cve_change_id": "5160FDEB-0FF0-457B-AA36-0AEDCAB2522E",
        "source_identifier": "nvd@nist.gov",
        "created": "2022-03-18T20:13:08.123",
        "details": [
            {
                "action": "Added",
                "type": "CVSS V2",
                "new_value": "NIST (AV:L/AC:L/Au:N/C:P/I:N/A:N)",
            },
            {
                "action": "Added",
                "type": "CVSS V3.1",
                "new_value": "NIST AV:L/AC:L/PR:L/UI:N/S:C/C:H/I:N/A:N",
            },
            {
                "action": "Changed",
                "type": "Reference Type",
                "old_value": "http://www.openwall.com/lists/oss-security/2022/03/18/2 No Types Assigned",
                "new_value": "http://www.openwall.com/lists/oss-security/2022/03/18/2 Mailing List, Third Party Advisory",
            },
        ],
    }

    if data:
        cve_changes.update(data)
    return cve_changes



# SPDX-FileCopyrightText: 2022-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from pontos.errors import PontosError


class GitHubScriptError(PontosError):
    """An error with a GitHub script"""



# Copyright (C) 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#


from pontos.errors import PontosError


class VersionError(PontosError):
    """
    Some error has occurred during version handling
    """


class ProjectError(PontosError):
    """
    An error has occured while gathering a project
    """



# SPDX-FileCopyrightText: 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

from dataclasses import dataclass
from datetime import datetime
from typing import Optional

from pontos.github.models.base import GitHubModel, User
from pontos.github.models.organization import Repository
from pontos.models import StrEnum


class AlertState(StrEnum):
    """
    State of a code scanning alert
    """

    OPEN = "open"
    DISMISSED = "dismissed"
    FIXED = "fixed"


class AlertSort(StrEnum):
    """
    The property by which to sort the alerts
    """

    CREATED = "created"
    UPDATED = "updated"


class DismissedReason(StrEnum):
    """
    The reason for dismissing or closing the alert
    """

    FALSE_POSITIVE = "false positive"
    WONT_FIX = "won't fix"
    USED_IN_TESTS = "used in tests"


class Severity(StrEnum):
    """
    The severity of the alert
    """

    NONE = "none"
    NOTE = "note"
    WARNING = "warning"
    ERROR = "error"


class SecuritySeverityLevel(StrEnum):
    """
    The security severity of the alert
    """

    LOW = "low"
    MEDIUM = "medium"
    HIGH = "high"
    CRITICAL = "critical"


class Classification(StrEnum):
    """
    A classification of the file. For example to identify it as generated
    """

    SOURCE = "source"
    GENERATED = "generated"
    TEST = "test"
    LIBRARY = "library"
    DOCUMENTATION = "documentation"


@dataclass
class Rule(GitHubModel):
    """
    A rule used to detect the alert

    Attributes:
        id: A unique identifier for the rule used to detect the alert
        name: The name of the rule used to detect the alert
        severity: The severity of the alert
        security_severity_level: The security severity of the alert
        description: A short description of the rule used to detect the alert
        full_description: description of the rule used to detect the alert
        tags: A set of tags applicable for the rule
        help: Detailed documentation for the rule as GitHub Flavored Markdown
        help_uri: A link to the documentation for the rule used to detect the
            alert
    """

    name: str
    description: str
    id: Optional[str] = None
    full_description: Optional[str] = None
    severity: Optional[Severity] = None
    security_severity_level: Optional[SecuritySeverityLevel] = None
    tags: Optional[list[str]] = None
    help: Optional[str] = None
    help_uri: Optional[str] = None


@dataclass
class Message(GitHubModel):
    """
    Attributes:
        text:
    """

    text: str


@dataclass
class Location(GitHubModel):
    """
    Describes a region within a file for the alert

    Attributes:
        path: The file path in the repository
        start_line: Line number at which the vulnerable code starts in the file
        end_line: Line number at which the vulnerable code ends in the file
        start_column: The column at which the vulnerable code starts within the
            start line
        end_column: The column at which the vulnerable code ends within the end
            line
    """

    path: str
    start_line: int
    end_line: int
    start_column: int
    end_column: int


@dataclass
class Instance(GitHubModel):
    """
    Attributes:
        ref: The full Git reference, formatted as `refs/heads/<branch name>`,
            `refs/pull/<number>/merge`, or `refs/pull/<number>/head`
        analysis_key: Identifies the configuration under which the analysis was
            executed. For example, in GitHub Actions this includes the workflow
            filename and job name
        environment: Identifies the variable values associated with the
            environment in which the analysis that generated this alert instance
            was performed, such as the language that was analyzed
        category: Identifies the configuration under which the analysis was
            executed. Used to distinguish between multiple analyses for the same
            tool and commit, but performed on different languages or different
            parts of the code
        state: State of a code scanning alert
        commit_sha:
        message:
        location: Describes a region within a file for the alert
        html_url:
        classifications: Classifications that have been applied to the file that
            triggered the alert. For example identifying it as documentation, or
            a generated file
    """

    ref: str
    analysis_key: str
    environment: str
    category: str
    state: AlertState
    commit_sha: str
    message: Message
    location: Location
    html_url: Optional[str] = None
    classifications: Optional[list[Classification]] = None


@dataclass
class Tool(GitHubModel):
    """
    A tool used to generate the code scanning analysis

    Attributes:
        name: The name of the tool used to generate the code scanning analysis
        version: The version of the tool used to generate the code scanning
            analysis
        guid: he GUID of the tool used to generate the code scanning analysis,
            if provided in the uploaded SARIF data
    """

    name: str
    version: Optional[str] = None
    guid: Optional[str] = None


@dataclass
class CodeScanningAlert(GitHubModel):
    """
    A GitHub Code Scanning Alert

    Attributes:
        number: The security alert number
        created_at: The time that the alert was created
        updated_at: The time that the alert was last updated
        url: The REST API URL of the alert resource
        html_url: The GitHub URL of the alert resource
        instances_url: The REST API URL for fetching the list of instances for
            an alert
        state: State of a code scanning alert
        fixed_at: The time that the alert was no longer detected and was
            considered fixed
        dismissed_by: A GitHub user who dismissed the alert
        dismissed_at: The time that the alert was dismissed
        dismissed_reason: The reason for dismissing or closing the alert
        dismissed_comment: The dismissal comment associated with the dismissal
            of the alert
        rule: The rule used to detect the alert
        tool: The tool used to generate the code scanning analysis
        most_recent_instance:
        repository: A GitHub repository
    """

    number: int
    created_at: datetime
    url: str
    html_url: str
    instances_url: str
    state: AlertState
    rule: Rule
    tool: Tool
    most_recent_instance: Instance
    repository: Optional[Repository] = None
    updated_at: Optional[datetime] = None
    fixed_at: Optional[datetime] = None
    dismissed_by: Optional[User] = None
    dismissed_at: Optional[datetime] = None
    dismissed_reason: Optional[DismissedReason] = None
    dismissed_comment: Optional[str] = None


@dataclass
class Analysis(GitHubModel):
    """
    Details for a code scanning analyses

    Attributes:
        ref: The full Git reference, formatted as `refs/heads/<branch name>`,
            `refs/pull/<number>/merge`, or `refs/pull/<number>/head`
        commit_sha: The SHA of the commit to which the analysis you are
            uploading relates
        analysis_key: Identifies the configuration under which the analysis was
            executed. For example, in GitHub Actions this includes the workflow
            filename and job name
        environment: Identifies the variable values associated with the
            environment in which this analysis was performed
        category: Identifies the configuration under which the analysis was
            executed. Used to distinguish between multiple analyses for the same
            tool and commit, but performed on different languages or different
            parts of the code
        error: Error generated when processing the analysis
        created_at: The time that the analysis was created
        results_count: The total number of results in the analysis
        rules_count: The total number of rules used in the analysis
        id: Unique identifier for this analysis
        url: The REST API URL of the analysis resource
        sarif_id: An identifier for the upload
        tool: The tool used to generate the code scanning analysis
        deletable:
        warning: Warning generated when processing the analysis
    """

    ref: str
    commit_sha: str
    analysis_key: str
    environment: str
    category: str
    error: str
    created_at: datetime
    results_count: int
    rules_count: int
    id: int
    url: str
    sarif_id: str
    tool: Tool
    deletable: bool
    warning: str


@dataclass
class CodeQLDatabase(GitHubModel):
    """
    A CodeQL database

    Attributes:
        id: The ID of the CodeQL database
        name: The name of the CodeQL database
        language: The language of the CodeQL database
        uploader: A GitHub user
        content_type: The MIME type of the CodeQL database file
        size: The size of the CodeQL database file in bytes
        created_at: The date and time at which the CodeQL database was created
        updated_at: The date and time at which the CodeQL database was last
            updated
        url: The URL at which to download the CodeQL database
        commit_oid: The commit SHA of the repository at the time the CodeQL
            database was created
    """

    id: int
    name: str
    language: str
    uploader: User
    content_type: str
    size: int
    created_at: datetime
    updated_at: datetime
    url: str
    commit_oid: Optional[str] = None


class DefaultSetupState(StrEnum):
    """
    State of a default setup
    """

    CONFIGURED = "configured"
    NOT_CONFIGURED = "not-configured"


class Language(StrEnum):
    """
    Analyzed Language
    """

    C_CPP = "c-cpp"
    CSHARP = "csharp"
    GO = "go"
    JAVA_KOTLIN = "java-kotlin"
    JAVASCRIPT_TYPESCRIPT = "javascript-typescript"
    JAVASCRIPT = "javascript"
    PYTHON = "python"
    RUBY = "ruby"
    TYPESCRIPT = "typescript"
    SWIFT = "swift"


class QuerySuite(StrEnum):
    """
    Used code scanning query suite
    """

    DEFAULT = "default"
    EXTENDED = "extended"


@dataclass
class DefaultSetup(GitHubModel):
    """
    Code scanning default setup configuration

    Attributes:
        state: Code scanning default setup has been configured or not
        languages: Languages to be analyzed
        query_suite: CodeQL query suite to be used
        updated_at: Timestamp of latest configuration update
        schedule: The frequency of the periodic analysis
    """

    state: DefaultSetupState
    languages: list[Language]
    query_suite: QuerySuite
    updated_at: Optional[datetime] = None
    schedule: Optional[str] = None


class SarifProcessingStatus(StrEnum):
    """
    `pending` files have not yet been processed, while `complete` means results
    from the SARIF have been stored. `failed` files have either not been
    processed at all, or could only be partially processed
    """

    PENDING = "pending"
    COMPLETE = "complete"
    FAILED = "failed"


@dataclass
class SarifUploadInformation(GitHubModel):
    """
    Information about the SARIF upload

    Attributes:
        processing_status: Status of the SARIF processing
        analyses_url: The REST API URL for getting the analyses associated with
            the upload
        errors: Any errors that ocurred during processing of the delivery
    """

    processing_status: SarifProcessingStatus
    analyses_url: Optional[str] = None
    errors: Optional[list[str]] = None



# Copyright (C) 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import unittest

from pontos.typing import SupportsStr


class SupportsStrTestCase(unittest.TestCase):
    def test_str(self):
        self.assertIsInstance("", SupportsStr)
        self.assertIsInstance(None, SupportsStr)

    def test_some_class(self):
        class Foo:
            def __str__(self) -> str:
                pass

        foo = Foo()

        self.assertIsInstance(foo, SupportsStr)
